if not nioConfig.LDB.CargoShip then return end
-- Load the cargoShip-library
local cargoShip = LibStub("LibCargoShip-2.1")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

--[[
	Option reference
	================
		name		The name of the data object to use

		parent		parent frame (default: UIParent)
		width		width of the block (updates automatically)
		height		height of the block (default: 12)
		scale		scale of the block (default: 1)
		alpha		alpha value of the block (default: 1)

		fontObject	e.g. GameFontNormalSmall (default: nil)
		font		a path to a font family (default: "Fonts\\FRIZQT__.TTF")
		fontSize	font size (default: 10)
		fontStyle	font style e.g. OUTLINE (default: nil)
		textColor	table holding color of the text (default: { 1, 1, 1 } = white)

		noShadow	if true, it hides the text shadow (default: nil)
		shadowX		shadow x-offset (default: 1)
		shadowY		shadow y-offset (default: -1)

		noIcon		if true, it hides the icon (default: nil)
		noText		if true, it hides the text (default: nil)
		useLabel	if true, it shows the dataobject's name in front of the value (default: nil)
		tagString	a tag string for formatting, e.g. "[name]: [value][suffix]"
]]

-- SetPoint Function
local function SetBrokerPosition(cargo, lastBroker, anchor, direction, x_off, y_off)
	if (not LDB:GetDataObjectByName(cargo.name)) then return lastBroker end

	cargo:SetPoint(anchor, lastBroker, direction, x_off, y_off)

	return cargo
end

local function InitBroker()
-- LEFT SIDE
-----------------------
local newPosition = CreateFrame"Frame"
newPosition:SetWidth(1)
newPosition:SetHeight(1)
newPosition:SetPoint("BOTTOMLEFT",2, 10.5)

local micro = cargoShip{
	name = "nioLDBMicro", 
}
newPosition = SetBrokerPosition(micro, newPosition, "LEFT", "RIGHT", 8, 0)

local wmoney = cargoShip{
	name = "Broker_wGold",
}
newPosition = SetBrokerPosition(wmoney, newPosition, "LEFT", "RIGHT", 8, 0)

local money = cargoShip{
	name = "nioLDBGold",
}
newPosition = SetBrokerPosition(money, newPosition, "LEFT", "RIGHT", 8, 0)

local currency = cargoShip{
	name = "CurrencyTracker",
	noIcon = false,
	tagString = "[text]",
}
newPosition = SetBrokerPosition(currency, newPosition, "LEFT", "RIGHT", 8, 0)

local XP = cargoShip{
	name = "nioLDBXP",
}
newPosition = SetBrokerPosition(XP, newPosition, "LEFT", "RIGHT", 8, 0)

local wdura = cargoShip{
	name = "Broker_wDurability",
}
newPosition = SetBrokerPosition(wdura, newPosition, "LEFT", "RIGHT", 8, 0)

local dura = cargoShip{
	name = "nioLDBDura",
}
newPosition = SetBrokerPosition(dura, newPosition, "LEFT", "RIGHT", 8, 0)

local bagsark = cargoShip{
	name = "ArkInventory_Bags",
}
newPosition = SetBrokerPosition(bagsark, newPosition, "LEFT", "RIGHT", 8, 0)

local bagsadi = cargoShip{
	name = "AdiBags",
}
newPosition = SetBrokerPosition(bagsadi, newPosition, "LEFT", "RIGHT", 8, 0)

local bagsync = cargoShip{
	name = "BagSyncLDB",
}
newPosition = SetBrokerPosition(bagsync, newPosition, "LEFT", "RIGHT", 8, 0)

local aratrade = cargoShip{
	name = "|cFFFFB366Ara|r Tradeskills",
	tagString = "Tradeskills",
}
newPosition = SetBrokerPosition(aratrade, newPosition, "LEFT", "RIGHT", 8, 0)

local trade = cargoShip{
	name = "nioLDBProfessions",
}
newPosition = SetBrokerPosition(trade, newPosition, "LEFT", "RIGHT", 8, 0)


local arafriends = cargoShip{
	name = "|cFFFFB366Ara|r Friends",
	tagString = "Friends:[text]",
}
newPosition = SetBrokerPosition(arafriends, newPosition, "LEFT", "RIGHT", 8, 0)

local friends = cargoShip{
	name = "nioLDBFriends",
}
newPosition = SetBrokerPosition(friends, newPosition, "LEFT", "RIGHT", 8, 0)


local guild = cargoShip{
	name = "|cFFFFB366Ara|r Guild",
	tagString = "Guild:[text]",
}
newPosition = SetBrokerPosition(guild, newPosition, "LEFT", "RIGHT", 8, 0)

local nioguild = cargoShip{
	name = "nioLDBGuild",
}
newPosition = SetBrokerPosition(nioguild, newPosition, "LEFT", "RIGHT", 8, 0)


-- MIDDLE
------------------------------
newPosition = CreateFrame"Frame"
newPosition:SetWidth(1)
newPosition:SetHeight(1)
newPosition:SetPoint("BOTTOM",-24, 10.5)

local ABS = cargoShip{
	name = "nioBroker_ActionBarSaver",
	tagString = "ABS",
}
newPosition = SetBrokerPosition(ABS, newPosition, "LEFT", "RIGHT", 8, 0)

local spec = cargoShip{
	name = "|cFFFFB366Ara|r SpecSwitcher",
}
newPosition = SetBrokerPosition(spec, newPosition, "LEFT", "RIGHT", 8, 0)

local equip = cargoShip{
	name = "Broker_Equipment", 
}
newPosition = SetBrokerPosition(equip, newPosition, "LEFT", "RIGHT", 8, 0)


-- RIGHT SIDE
------------------------------
newPosition = CreateFrame"Frame"
newPosition:SetWidth(1)
newPosition:SetHeight(1)
newPosition:SetPoint("BOTTOMRIGHT",0, 10.5)

local wclock = cargoShip{
	name = "Broker_wClock",
}
newPosition = SetBrokerPosition(wclock, newPosition, "RIGHT", "LEFT", -12, 0)

local nclock = cargoShip{
	name = "nioLDBClock",
}
newPosition = SetBrokerPosition(nclock, newPosition, "RIGHT", "LEFT", -12, 0)

local diffi = cargoShip{
	name = "V_Difficulty",
}
newPosition = SetBrokerPosition(diffi, newPosition, "RIGHT", "LEFT", -12, 0)

local bug = cargoShip{
	name = "BugSack",
}
newPosition = SetBrokerPosition(bug, newPosition, "RIGHT", "LEFT", -12, 0)

local shlatenz = cargoShip{
	name = "shLatency",
	tagString = "[text]",
}
newPosition = SetBrokerPosition(shlatenz, newPosition, "RIGHT", "LEFT", -12, 0)

local latenz = cargoShip{
	name = "nioLDBLatency",
	tagString = "[text]",
}
newPosition = SetBrokerPosition(latenz, newPosition, "RIGHT", "LEFT", -12, 0)

local shfps = cargoShip{
	name = "shFps",
}
newPosition = SetBrokerPosition(shfps, newPosition, "RIGHT", "LEFT", -12, 0)

local fps = cargoShip{
	name = "nioLDBFPS",
}
newPosition = SetBrokerPosition(fps, newPosition, "RIGHT", "LEFT", -12, 0)

-- local WAuras = cargoShip{
	-- name = "WeakAuras",
	-- noIcon = false,
	-- tagString = "WA",
-- }
-- newPosition = SetBrokerPosition(WAuras, newPosition, "RIGHT", "LEFT", -12, 0)

local bskada = cargoShip{
	name = "Skada",
	tagString = "Skada",
}
newPosition = SetBrokerPosition(bskada, newPosition, "RIGHT", "LEFT", -12, 0)

local numeration = cargoShip{
	name = "Numeration",
}
newPosition = SetBrokerPosition(numeration, newPosition, "RIGHT", "LEFT", -12, 0)

-- local batlasloot = cargoShip{
	-- name = "AtlasLoot",
	-- tagString = "ALoot",
-- }
-- newPosition = SetBrokerPosition(batlasloot, newPosition, "RIGHT", "LEFT", -12, 0)

-- local bDBM = cargoShip{
	-- name = "DBM-LDB",
	-- tagString = "DBM",
-- }
-- newPosition = SetBrokerPosition(bDBM, newPosition, "RIGHT", "LEFT", -12, 0)

local petbattle = cargoShip{
	name = "nioLDBPetBattle",
}
newPosition = SetBrokerPosition(petbattle, newPosition, "RIGHT", "LEFT", -12, 0)

local Raid = cargoShip{
	name = "nioLDBRaid",
	tagString = "Raid",
}
newPosition = SetBrokerPosition(Raid, newPosition, "RIGHT", "LEFT", -12, 0)

local Addons = cargoShip{
	name = "nioLDBAddons",
	tagString = "Addons",
}
newPosition = SetBrokerPosition(Addons, newPosition, "RIGHT", "LEFT", -12, 0)

end

local nioBroker = CreateFrame"Frame"
nioBroker:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		InitBroker()
		nioBroker:UnregisterEvent("PLAYER_ENTERING_WORLD")
		nioBroker = nil
	end	
end)
nioBroker:RegisterEvent("PLAYER_ENTERING_WORLD")
