local LDB = LibStub('LibDataBroker-1.1')

local function OnEventWrapper(events, eventfunction)
	local frame = CreateFrame("frame")
	for _,e in ipairs{strsplit(" ",events)} do frame:RegisterEvent(e) end
	frame:SetScript("OnEvent", eventfunction) 
end

local function OnUpdateWrapper(ldbobj, eventfunction)
	local frame = CreateFrame("frame")
	frame:RegisterEvent("OnUpdate")
	
	frame:SetScript("OnUpdate", function(self, u)
		ldbobj.elapsed = ldbobj.elapsed + u
		if ldbobj.elapsed > ldbobj.update then 
			ldbobj.text = eventfunction()
			ldbobj.elapsed = 0 
		end	
	end)
end

----------------------
--      Make LDBs     --
----------------------
local cfg = nioConfig.LDB

--Micro Menu
if cfg.MicroMenu then
	local Micro = LDB:NewDataObject('nioLDBMicro', { 
		type = 'data source',
		text = 'MicroMenu',
		icon = [[Interface\GROUPFRAME\UI-Group-LeaderIcon]],
		MenuEntries = {
			{
				text = MAINMENU_BUTTON,
				isTitle = true,
				notCheckable = true,
			},
			{
				text = CHARACTER_BUTTON,
				func = function() 
					securecall(ToggleCharacter, 'PaperDollFrame') 
				end,
				notCheckable = true,
			},
			{
				text = SPELLBOOK_ABILITIES_BUTTON,
				func = function() 
					securecall(ToggleSpellBook, BOOKTYPE_SPELL)
				end,
				notCheckable = true,
			},
			{
				text = TALENTS_BUTTON,
				func = function() 
					if not PlayerTalentFrame then 
						LoadAddOn('Blizzard_TalentUI') 
					end
					if not GlyphFrame then 
						LoadAddOn('Blizzard_GlyphUI') 
					end 
					PlayerTalentFrame_Toggle()
				end,
				notCheckable = true,
			},
			{
				text = PET_JOURNAL,
				func = function() 
					securecall(TogglePetJournal) 
				end,
				notCheckable = true,
			},
			{
				text = BACKPACK_TOOLTIP,
				func = function() 
					securecall(ToggleAllBags) 
				end,
				notCheckable = true,
			},
			{
				text = " ",
				notCheckable = true,
			},
			{
				text = ACHIEVEMENT_BUTTON,
				func = function() 
					securecall(ToggleAchievementFrame) 
				end,
				notCheckable = true,
			},
			{
				text = QUESTLOG_BUTTON,
				func = function() 
					securecall(ToggleFrame, QuestLogFrame) 
				end,
				notCheckable = true,
			},
			{
				text = RAID,
				func = function() 
					securecall(ToggleFriendsFrame, 4)
				end,
				notCheckable = true,
			},
			{
				text = ENCOUNTER_JOURNAL,
				func = function() 
					securecall(ToggleEncounterJournal)
				end,
				notCheckable = true,
			},
			{
				text = " ",
				notCheckable = true,
			},		
			{
				text = GUILD,
				arg1 = IsInGuild('player'),
				func = function() 
					ToggleGuildFrame()
				end,
				notCheckable = true,
			},
			{
				text = SOCIAL_BUTTON,
				func = function() 
					securecall(ToggleFriendsFrame, 1) 
				end,
				notCheckable = true,
			},
			{
				text = GM_EMAIL_NAME,
				func = function() 
					securecall(ToggleHelpFrame) 
				end,
				notCheckable = true,
			},
			{
				text = " ",
				notCheckable = true,
			},	
			{
				text = DUNGEONS_BUTTON,
				func = function() 
					securecall(ToggleLFDParentFrame)
				end,
				notCheckable = true,
			},
			{
				text = PLAYER_V_PLAYER,
				func = function() 
					securecall(ToggleFrame, PVPFrame) 
				end,
				notCheckable = true,
			},
			{
				text = " ",
				notCheckable = true,
			},	
			{
				text = KEY_BINDINGS,
				func = function()
					if not KeyBindingFrame then 
						LoadAddOn('Blizzard_BindingUI') 
					end
					if not KeyBindingFrame:IsVisible() then
						ShowUIPanel(KeyBindingFrame)
						KeyBindingFrame.ignoreMenu = true
					else
						HideUIPanel(KeyBindingFrame)
					end
				end,
				notCheckable = true,
			},
			{
				text = MACROS,
				func = function()
					if not MacroFrame then 
						LoadAddOn('Blizzard_MacroUI') 
					end
					if not MacroFrame:IsVisible() then
						ShowUIPanel(MacroFrame)
					else
						HideUIPanel(MacroFrame)
					end
				end,
				notCheckable = true,
			},
		},
	})
	function Micro.OnEnter(frame)
		local menuFrameMicro = CreateFrame('Frame', 'niomenuDropDownMenuMicro', UIParent, 'UIDropDownMenuTemplate')
		EasyMenu(Micro.MenuEntries, menuFrameMicro, frame, 0, 0, 'MENU', 8)
	end
end

--XP
if cfg.XP and UnitLevel("player") < 90 then	
	local LDBXP = LDB:NewDataObject('nioLDBXP', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = "99%", 
	})
	--local start, cur, max, starttime, startlevel
	LDBXP.OnEvent = function(self,event)
		if (event=="PLAYER_LOGIN") then
			LDBXP.start, LDBXP.maxi, LDBXP.starttime = UnitXP("player"), UnitXPMax("player"), GetTime()
			LDBXP.cur = LDBXP.start
			LDBXP.startlevel = UnitLevel("player") + LDBXP.start/LDBXP.maxi
			LDBXP.text = string.format("%d%%", LDBXP.cur/LDBXP.maxi*100)
			self:UnregisterEvent("PLAYER_LOGIN")
		elseif (event=="PLAYER_XP_UPDATE") then
			LDBXP.cur = UnitXP("player")
			LDBXP.maxi = UnitXPMax("player")
			LDBXP.text = string.format("%d%%", LDBXP.cur/LDBXP.maxi*100)	
		elseif (event=="PLAYER_LEVEL_UP") then
			LDBXP.start = LDBXP.start - LDBXP.maxi
		end
	end
	LDBXP.OnTooltipShow = function(tooltip)
		tooltip:ClearLines()
		tooltip:AddLine("XP")
		tooltip:AddDoubleLine("XP:", LDBXP.cur.."/"..LDBXP.maxi, nil,nil,nil, 1,1,1)
		tooltip:AddDoubleLine("Rest:", string.format("%d%%", (GetXPExhaustion() or 0)/LDBXP.maxi*100), nil,nil,nil, 1,1,1)
		tooltip:AddDoubleLine("TNL:", LDBXP.maxi-LDBXP.cur, nil,nil,nil, 1,1,1)
		tooltip:AddLine(string.format("%.2f hours played this session", (GetTime()-LDBXP.starttime)/3600), 1,1,1)
		tooltip:AddLine((LDBXP.cur - LDBXP.start).." XP gained this session", 1,1,1)
		tooltip:AddLine(string.format("%.2f levels gained this session", UnitLevel("player") + LDBXP.cur/LDBXP.maxi - LDBXP.startlevel), 1,1,1)
	end
	OnEventWrapper("PLAYER_LOGIN PLAYER_XP_UPDATE PLAYER_LEVEL_UP", LDBXP.OnEvent)
end

--LDB 2
if cfg.Addons then
	local Addons = LDB:NewDataObject('nioLDBAddons', { 
		type = 'data source',
		text = 'Addons',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		MenuEntries = {
			{
			text = "nioUI",
			icon = "Interface\\Icons\\Achievement_dungeon_nexus70_normal",
			func = function() 
				InterfaceOptionsFrame_OpenToCategory("nioUI")
			end,
			notCheckable = true,
			},
		},
	})
	if IsAddOnLoaded("DBM-Core") then table.insert(Addons.MenuEntries, {
		text = "DBM",
		icon = "Interface\\Icons\\Ability_CheapShot",
		func = function() 
				DBM:LoadGUI()
			end,
		notCheckable = true,
	})
	end
	if IsAddOnLoaded("WeakAuras") then table.insert(Addons.MenuEntries, {
		text = "WeakAuras",
		icon = "Interface\\AddOns\\WeakAuras\\icon.tga",
		func = function() 
				SlashCmdList.WEAKAURAS()
			end,
		notCheckable = true,
	})
	end
	if IsAddOnLoaded("AtlasLoot") then table.insert(Addons.MenuEntries, {
		text = "AtlasLoot",
		icon = "Interface\\Icons\\INV_Box_01",
		func = function() 
				AtlasLoot:ShowFrame_MiniMap()
			end,
		notCheckable = true,
	})
	end
	function Addons.OnClick(frame)
		local menuFrameAddons = CreateFrame('Frame', 'niomenuDropDownMenuAddons', UIParent, 'UIDropDownMenuTemplate')
		EasyMenu(Addons.MenuEntries, menuFrameAddons, frame, 0, 0, 'MENU', 8)
	end
end

--LDB 3
if cfg.Raid then
	local Raid = LDB:NewDataObject('nioLDBRaid', { 
		type = 'data source',
		text = 'Raid',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		MenuEntries = {
			{
				text = "TS",
				icon = "Interface\\Icons\\Ability_BullRush",
				func = function() 
						local Channel = "PARTY"
						if IsInRaid() then Channel = "RAID" end
						SendChatMessage("-------- "..nioConfig.TS.Program.." ---------", Channel)
						SendChatMessage("IP: "..nioConfig.TS.Data, Channel)
						SendChatMessage("PW: "..nioConfig.TS.PW, Channel)
						SendChatMessage("------------------------------", Channel)
					end,
				notCheckable = true,
			},
			{
				text = "Rolecheck",
				icon = "Interface\\Icons\\Ability_Creature_Cursed_01",
				func = function() 
						InitiateRolePoll() 
					end,
				notCheckable = true,
			},
			{
				text = "Readycheck",
				icon = "Interface\\Icons\\Ability_Creature_Cursed_01",
				func = function() 
						DoReadyCheck()
					end,
				notCheckable = true,
			},
		},
	})
	if IsAddOnLoaded("MarkAssistant") then 
		table.insert(Raid.MenuEntries, 
		{
			text = "Show MA",
			icon = "Interface\\Icons\\Ability_Hunter_FocusedAim",
			func = function() 
					SlashCmdList.MarkAssistant("show") 
				end,
			notCheckable = true,
		})
		table.insert(Raid.MenuEntries, 
		{
			text = "Hide MA",
			icon = "Interface\\Icons\\Ability_Hunter_FocusedAim",
			func = function() 
					SlashCmdList.MarkAssistant("hide")  
				end,
			notCheckable = true,
		})
	end
	function Raid.OnClick(frame)
		local menuFrameRaid = CreateFrame('Frame', 'niomenuDropDownMenuRaid', UIParent, 'UIDropDownMenuTemplate')
		EasyMenu(Raid.MenuEntries, menuFrameRaid, frame, 0, 0, 'MENU', 8)
	end
end

-- --clock
if cfg.Clock and not IsAddOnLoaded("Broker_wClock") then
	local LDBClock = LDB:NewDataObject('nioLDBClock', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = '',
		elapsed = 0,
		update = 5,
		OnClick = function(self, button)
			if (button=="RightButton") then
				if not (IsAddOnLoaded("Blizzard_TimeManager")) then
					UIParentLoadAddOn("Blizzard_TimeManager")
					ToggleTimeManager()
				else
					ToggleTimeManager()
				end
			elseif (button=="LeftButton") then
				if (not IsAddOnLoaded("Blizzard_Calendar")) then
					UIParentLoadAddOn("Blizzard_Calendar")
					Calendar_Toggle()
				else
					Calendar_Toggle()
				end
			end
		end,
		OnTooltipShow = function(tooltip)
			local dateInfo = date("*t")
			local minit = dateInfo.min
			if minit < 10 then minit = ("0"..minit) end
			local LocalTime = dateInfo.hour..":"..minit
			
			dateInfo, minit = GetGameTime()
			if minit < 10 then minit = ("0"..minit) end
			local ServerTime = dateInfo..":"..minit
		
			tooltip:AddLine("Time")
			tooltip:AddDoubleLine("Server time:",(ServerTime),nil,nil,nil,1,1,1)
			tooltip:AddDoubleLine("Local time:",(LocalTime),nil,nil,nil,1,1,1)	
			tooltip:AddLine(" ")
			tooltip:AddLine("Options")
			tooltip:AddLine("|cff69ccf0Click|cffffd200 to toggle calendar")
			tooltip:AddLine("|cff69ccf0Right-Click|cffffd200 to toggle time manager")
		end,
	})
	LDBClock.OnUpdate = function(self, u)		
		local hour, minit
		if cfg.ClockLocal then 
			hour = date("*t")
			minit = hour.min
			hour = hour.hour
		else
			hour, minit = GetGameTime()
		end
		if minit < 10 then minit = ("0"..minit) end
		return hour..":"..minit
	end
	OnUpdateWrapper(LDBClock, LDBClock.OnUpdate)
end

--durability
if cfg.Dura and not IsAddOnLoaded("Broker_wDurability") then
	local LDBDura = LDB:NewDataObject('nioLDBDura', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = '',
	})
	LDBDura.OnEvent = function(self, event)
		local dmin = 100
		for id = 1, 18 do
			local dur, dmax = GetInventoryItemDurability(id)
			if dur ~= dmax then dmin = floor(min(dmin,dur/dmax*100)) end
		end
		LDBDura.text = "Dur:"..dmin
	end
	OnEventWrapper("PLAYER_LOGIN UPDATE_INVENTORY_DURABILITY", LDBDura.OnEvent)
end

--Professions
if cfg.Professions then
	local primary = {
		(GetSpellInfo(2259)), -- Alchemy
		(GetSpellInfo(2018)), -- Blacksmithing
		(GetSpellInfo(7411)), -- Enchanting
		(GetSpellInfo(4036)), -- Engineering
		(GetSpellInfo(25229)), -- Jewelcrafting
		(GetSpellInfo(2108)), -- Leatherworking
		(GetSpellInfo(2656)), -- Smelting
		(GetSpellInfo(3908)), -- Tailoring
		(GetSpellInfo(53428)), -- Runeforging
		(GetSpellInfo(45357)), -- Inscription
	}
	local secondary = {
		(GetSpellInfo(78670)), -- Archaeology
		(GetSpellInfo(2550)), -- Cooking
		(GetSpellInfo(3273)), -- First Aid
	}
	local LDBProfessions = LDB:NewDataObject('nioLDBProfessions', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = 'Tradeskills',
		MenuEntries = {},
	})
	function LDBProfessions.OnClick(frame)
		local menuFrameLDBProfessions = CreateFrame('Frame', 'niomenuDropDownMenuLDBProfessions', UIParent, 'UIDropDownMenuTemplate')
		LDBProfessions.MenuEntries = {}
		local proficon		
		for k,v in pairs(primary) do
			proficon = select(3, GetSpellInfo(v))
			if proficon then
				table.insert(LDBProfessions.MenuEntries, 
				{
					text = v,
					icon = proficon,
					func = function() 
							CastSpellByName(v)
						end,
					notCheckable = true,
				})
			end			
		end		
		for k,v in pairs(secondary) do
			proficon = select(3, GetSpellInfo(v))
			if proficon then
				table.insert(LDBProfessions.MenuEntries, 
				{
					text = v,
					icon = proficon,
					func = function() 
							CastSpellByName(v)
						end,
					notCheckable = true,
				})
			end
		end		
		EasyMenu(LDBProfessions.MenuEntries, menuFrameLDBProfessions, frame, 0, 0, 'MENU', 8)
	end
end

-- Guild
if cfg.Guild or cfg.Friends then
	local function RGB2Hex(c)
		return format("%02x%02x%02x", c.r*255, c.g*255, c.b*255)
	end
	--FriendsFont_Normal:SetFont("Interface\\AddOns\\nioUI\\fonts\\ProFontWindows.ttf", 10,nil)
	nioGuildFont = CreateFont("nioGuildFont")
	nioGuildFont:SetFont("Interface\\AddOns\\nioUI\\fonts\\ProFontWindows.ttf", 10,nil)
	
	--stupid special characters....
	local function chsize(char)
		if not char then return 0
		elseif char > 240 then return 4
		elseif char > 225 then return 3
		elseif char > 192 then return 2
		else return 1
		end
	end	
	--stupid special characters....
	local function nioGetPlayerName(name, status, Spaces)
		local currentIndex = 1
		while currentIndex <= #name do
			local char = string.byte(name, currentIndex)
			Spaces = Spaces + chsize(char) -1
			currentIndex = currentIndex + chsize(char)
		end
		return format("%-"..Spaces.."s",name..status) 
	end
	
	if cfg.Guild then
		local function Guild2Color(class)
			local c = _G["RAID_CLASS_COLORS"][class]
			return RGB2Hex(c)
		end
		local LDBGuild = LDB:NewDataObject('nioLDBGuild', { 
			type = 'data source',
			icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
			text = 'Guild',
			MenuEntries = {},
		})
		function LDBGuild.OnClick(frame, button)
			if not IsInGuild() then return end
					
			if button == "LeftButton" then
				local menuFrameGuild = CreateFrame('Frame', 'niomenuDropDownMenuGuild', UIParent, 'UIDropDownMenuTemplate')
				LDBGuild.MenuEntries = {}
				GuildRoster()
				local total, online = GetNumGuildMembers()	
				
				table.insert(LDBGuild.MenuEntries, {
					text = "|cff0C82CC"..GetGuildLevel().."  "..GetGuildInfo("player").."|r",
					notCheckable = true,	
					fontObject = "FriendsFont_Large",
					func = function() 
						ToggleGuildFrame()
					end,
				})	
				
				table.insert(LDBGuild.MenuEntries, {
					text = "|cffFFCC00"..GetGuildRosterMOTD().."|r",
					notCheckable = true,	
				})	
				
				for i = 1, total do
					local name, rank, _, level, class, zone, note, officernote, isonline, status, trueClass = GetGuildRosterInfo(i)		
					
					if isonline then			
						local difficultyColor = RGB2Hex(GetQuestDifficultyColor(level))
						local guildRankText = "|cff19E6D8"..rank.."|r"
						local classColor = Guild2Color(trueClass)
						local groupIndicator = "|cff0fff0f |r"
						local zoneColor = "|cffffffff"

						if UnitInParty(name) or UnitInRaid(name) then
							groupIndicator = "|cff0fff0f+|r"
						end

						if note ~= "" then
							note = "|cff24C226"..note.."|r"
						end

						if CanViewOfficerNote() and officernote ~= "" then
							note = note.."|cffFF8F40 - "..officernote.."|r"
						end

						if zone and (zone == GetRealZoneText() or zone == GetMapNameByID(GetCurrentMapAreaID())) then
							zoneColor = "|cff00FF00"
						else
							zoneColor = "|cffFFFF00"
						end
						zone = zone or _G["UNKNOWN"]

						if status == 0 then
							status = " "
						elseif status == 1 then
							status = "<"..DEFAULT_AFK_MESSAGE..">"
						elseif status == 2 then
							status = "<"..DEFAULT_DND_MESSAGE..">"
						end

						table.insert(LDBGuild.MenuEntries, {
							text = groupIndicator.." |cff"..difficultyColor..format("%-2s",level).."|r".." |cff"..classColor..nioGetPlayerName(name, status, 20).."|r"..zoneColor..format("%-35s",zone).."|r".."|cff19E6D8"..format("%-20s",rank).."|r".."|cff24C226"..format("%-20s",note).."|r",
							fontObject = "nioGuildFont",
							func = function() 
								if IsAltKeyDown() then
									InviteUnit(name)
								else
									SetItemRef( "player:"..name, ("|Hplayer:%1$s|h[%1$s]|h"):format(name), "LeftButton" )
								end 	
							end,
							notCheckable = true,
						})	
					end
				end	
			
				table.insert(LDBGuild.MenuEntries, {
					text = "\n|cff69ccf0Click|cffffd200 to whisper player",
					notCheckable = true,	
				})	
				table.insert(LDBGuild.MenuEntries, {
					text = "|cff69ccf0Alt-Click|cffffd200 to invite player",
					notCheckable = true,	
				})	
				
				EasyMenu(LDBGuild.MenuEntries, menuFrameGuild, frame, -200, 50, 'MENU', 8)			
			else
				ToggleGuildFrame()
			end
		end
		LDBGuild.OnEvent = function(self,event)
			if not IsInGuild() then return end
			local total, online = GetNumGuildMembers()
			LDBGuild.text = format("Guild:%s/%s", online, total)
		end	
		OnEventWrapper("PLAYER_GUILD_UPDATE GUILD_ROSTER_UPDATE", LDBGuild.OnEvent)
	end
	
	if cfg.Friends then
		local function Friend2Color(class)
			local result
			for k,v in pairs(_G["LOCALIZED_CLASS_NAMES_FEMALE"]) do
				if v == class then
					result = k
				end
			end
			if not result then
				for k,v in pairs(_G["LOCALIZED_CLASS_NAMES_MALE"]) do
					if v == class then
						result = k
					end
				end
			end
			local c = _G["RAID_CLASS_COLORS"][result]
			return RGB2Hex(c)
		end
		local LDBFriends = LDB:NewDataObject('nioLDBFriends', { 
			type = 'data source',
			icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
			text = 'Friends',
			MenuEntries = {},
		})
		function LDBFriends.OnClick(frame, button)					
			if button == "LeftButton" then			
				local menuFrameLDBFriends = CreateFrame('Frame', 'niomenuDropDownMenuLDBFriends', UIParent, 'UIDropDownMenuTemplate')
				LDBFriends.MenuEntries = {}

				local bnTotal, bnCount = BNGetNumFriends()
				local fTotal, fCount = GetNumFriends()
			
				if bnTotal and bnTotal > 0 then
					table.insert(LDBFriends.MenuEntries, {
						text = "|cffFFFF00RealID|r",
						notCheckable = true,
						fontObject = "FriendsFont_Large",						
					})						
					for i=1, bnCount do
						local presenceID, name, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, isAfk, isDnd, broadcast, note = BNGetFriendInfo(i)
						if isOnline then
							local wname
							local _, _, client, realmName, realmID, faction, race, class, guild, zoneName, level, gameText = BNGetToonInfo(toonID or 0)	
							status = ""
							if isAfk then
								status = "<"..DEFAULT_AFK_MESSAGE..">"
							elseif isDnd then
								status = "<"..DEFAULT_DND_MESSAGE..">"
							end
							
							if client == "WoW" then
								local difficultyColor = RGB2Hex(GetQuestDifficultyColor(tonumber(level) or 1))
								local classColor = Friend2Color(class)
								local groupIndicator = "|cff0fff0f |r"
								local zoneColor = "|cffffffff"

								note = note or ""
								zoneName = zoneName or _G["UNKNOWN"]

								if realmName == GetRealmName() then
									if UnitInParty(toonName) or UnitInRaid(toonName) then
										groupIndicator = "|cff0fff0f+|r"
									end
									if zoneName and (zoneName == GetRealZoneText() or zoneName == GetMapNameByID(GetCurrentMapAreaID())) then
										zoneColor = "|cff00FF00"
									else
										zoneColor = "|cffFFFF00"
									end
								else
									zoneColor = "|cff0fff0f"
									zoneName = realmName
								end										

								table.insert(LDBFriends.MenuEntries, {
									text = groupIndicator.."|cff19E6D8"..format("%-40s",name.." ").."|r".." |cff"..difficultyColor..format("%-2s",level).."|r".." |cff"..classColor..nioGetPlayerName(toonName, status, 20).."|r"..zoneColor..format("%-40s",zoneName).."|r".."|cff24C226"..note.."|r",
									fontObject = "nioGuildFont",
									func = function() 
										if IsAltKeyDown() then
											InviteUnit(toonName)
										else
											if (name) then
												wname = ("%s:%d"):format(name, presenceID)
												_G.SetItemRef(("BNplayer:%s"):format(wname), ("|HBNplayer:%s|h[%s]|h"):format(wname, name), "LeftButton");
											else
												wname = presenceID and name or toonName
												SetItemRef( "player:"..wname, ("|Hplayer:%1$s|h[%1$s]|h"):format(wname), "LeftButton" )
											end
										end 	
									end,
									notCheckable = true,
								})	
							elseif client == "S2" then
								note = note or ""
								wname = ("%s:%d"):format(name, presenceID)
								table.insert(LDBFriends.MenuEntries, {
									text = 	format("%-40s",name.." "..status).."|cff0C82CC"..format("%-15s","Starcraft 2").."|r".."|cffFFFF00"..format("%-40s",gameText).."|r".."|cff24C226"..note.."|r",	
									notCheckable = true,
									func = function()
										_G.SetItemRef(("BNplayer:%s"):format(wname), ("|HBNplayer:%s|h[%s]|h"):format(wname, name), "LeftButton");
									end
								})
							elseif client == "D3" then
								note = note or ""
								wname = ("%s:%d"):format(name, presenceID)
								table.insert(LDBFriends.MenuEntries, {
									text = 	format("%-40s",name.." "..status).."|cffFFFF00"..format("%-15s","Diablo 3").."|r".."|cffFFFF00"..format("%-25s",gameText).."|r".."|cff24C226"..note.."|r",	
									notCheckable = true,
									func = function()
										_G.SetItemRef(("BNplayer:%s"):format(wname), ("|HBNplayer:%s|h[%s]|h"):format(wname, name), "LeftButton");
									end
								})
							end
						end
					end					
				end
				
				if fTotal > 0 then
					table.insert(LDBFriends.MenuEntries, {
					text = "|cff00FF00Friends|r",
					fontObject = "FriendsFont_Large",
					notCheckable = true,	
					})	
					for i = 1, fCount do
						local name, level, class, zone, _, status, note = GetFriendInfo(i)
						local difficultyColor = RGB2Hex(GetQuestDifficultyColor(level))
						local classColor = Friend2Color(class)
						local groupIndicator = "|cff0fff0f |r"
						local zoneColor = " |cffffffff"

						if UnitInParty(name) or UnitInRaid(name) then
							groupIndicator = "|cff0fff0f+|r"
						end

						note = note or ""
						zone = zone or _G["UNKNOWN"]
						
						if status == 0 then
							status = " "
						elseif status == 1 then
							status = "<"..DEFAULT_AFK_MESSAGE..">"
						elseif status == 2 then
							status = "<"..DEFAULT_DND_MESSAGE..">"
						end

						if zone and (zone == GetRealZoneText() or zone == GetMapNameByID(GetCurrentMapAreaID())) then
							zoneColor = "|cff00FF00"
						else
							zoneColor = "|cffFFFF00"
						end
						table.insert(LDBFriends.MenuEntries, {
							text = groupIndicator.." |cff"..difficultyColor..format("%-2s",level).."|r".." |cff"..classColor..nioGetPlayerName(name, status, 20).."|r"..zoneColor..format("%-40s",zone).."|r".."|cff24C226"..note.."|r",
							fontObject = "nioGuildFont",
							func = function() 
								if IsAltKeyDown() then
									InviteUnit(name)
								else
									SetItemRef( "player:"..name, ("|Hplayer:%1$s|h[%1$s]|h"):format(name), "LeftButton" )
								end 	
							end,
							notCheckable = true,
						})	
					end
				end		
				
				table.insert(LDBFriends.MenuEntries, {
					text = "\n|cff69ccf0Click|cffffd200 to whisper player",
					notCheckable = true,	
				})	
				table.insert(LDBFriends.MenuEntries, {
					text = "|cff69ccf0Alt-Click|cffffd200 to invite player",
					notCheckable = true,	
				})	
				
				EasyMenu(LDBFriends.MenuEntries, menuFrameLDBFriends, frame, 0, 0, 'MENU', 1)					
			else
				ToggleFriendsFrame(1)
			end
		end
		LDBFriends.OnEvent = function(self,event)
			local bnTotal, bnCount = BNGetNumFriends()
			local fTotal, fCount = GetNumFriends()
			LDBFriends.text = format("Friends:%d/%d", bnCount+fCount, bnTotal+fTotal)
			if event == "PLAYER_ENTERING_WORLD" then self:UnregisterEvent("PLAYER_ENTERING_WORLD") end
		end	
		OnEventWrapper("FRIENDLIST_UPDATE PLAYER_ENTERING_WORLD", LDBFriends.OnEvent)
	end
end

--fps
if cfg.FPS then
	local LDBFPS = LDB:NewDataObject('nioLDBFPS', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = '',
		elapsed = 0,
		update = 2,
	})
	LDBFPS.OnUpdate = function()		
		return floor(GetFramerate()).." fps"
	end
	OnUpdateWrapper(LDBFPS, LDBFPS.OnUpdate)
end

--latency
if cfg.Latency then
	local LDBLatency = LDB:NewDataObject('nioLDBLatency', { 
		type = 'data source',
		icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
		text = '',
		elapsed = 0,
		update = 2,
	})
	LDBLatency.OnUpdate = function()		
		local _,_,home,world = GetNetStats()
		return home.." ms/"..world.." ms" 
	end
	OnUpdateWrapper(LDBLatency, LDBLatency.OnUpdate)
	
	
	-- Memory (from shperformance)
	if cfg.Memory then	
		local colorgradient = 50 
		local prevmem, tipshownMem, tipshownLatency = collectgarbage("count")
		local addons, tester, name = {} 
		for i=1,GetNumAddOns(), 1 do
			tester = select(4,GetAddOnInfo(i))
			if  tester == 1 then -->check to see if addon is even enabled 
				name = select(1,GetAddOnInfo(i))
				table.insert(addons, name)
			end 
		end
		sort(addons, function(a,b) return a and b and a:lower() < b:lower() end)
		-->Format Mem with stylez
		local formatMem = function(mem, x)
			if x then
				if mem > 1024 then return format("%.2f |cffE8D200mb|r", mem/1024)
				else return format("%.1f |cffE8D200kb|r", mem) end
			else
				if mem > 1024 then return format("%.2f mb", mem/1024)
				else return format("%.1f kb", mem) end
			end
		end	
		-->TEKKUB created function for color gradience, awesome!--
		local ColorGradient = function(perc, r1, g1, b1, r2, g2, b2, r3, g3, b3)
			if perc >= 1 then return r3, g3, b3 elseif perc <= 0 then return r1, g1, b1 end
			local segment, relperc = math.modf(perc*2)
			if segment == 1 then r1, g1, b1, r2, g2, b2 = r2, g2, b2, r3, g3, b3 end
			return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
		end		
		LDBLatency.OnTooltipShow = function(tooltip)
			tooltip:ClearLines()
			
			tooltip:AddLine("[Memory]")			
			tooltip:AddDoubleLine("# | |cffC3771AAddon|r |cff06ddfaName|r", format("|cffC3771AMemory|r above (|cff06ddfa%s kb|r)", 60))
			tooltip:AddDoubleLine("|cffffffff------------|r", "|cffffffff------------|r")
			
			local counter = 0 -- for numbering (listing) and coloring
			local addonmem = 0
			local hidden, hiddenmem, shownmem = 0, 0, 0
			
			UpdateAddOnMemoryUsage()
			local newname
			for i, v in ipairs(addons) do
				local mem = GetAddOnMemoryUsage(v)
				local r, g, b = ColorGradient((mem - 60)/1024, 0,1,0, 1,1,0, 1,0,0)
				addonmem = addonmem + mem
				if mem > 60  and 100 > counter then
					counter = counter + 1
					hidden = #addons - counter
					shownmem = shownmem + mem
					newname = select(2,GetAddOnInfo(v))
					local memstr = formatMem(mem) 

					if counter < 10 then tooltip:AddDoubleLine(format("  |cffDAB024%.0f)|r %s", counter, newname), memstr, 1, 1, 1, r, g, b)  
					else tooltip:AddDoubleLine(format("|cffDAB024%.0f)|r %s", counter, newname), memstr, 1, 1, 1, r, g, b) end
				end
			end
			
			tooltip:AddDoubleLine("|cffffffff------------|r", " ")
			
			hiddenmem = addonmem - shownmem
			local r, g, b = ColorGradient(addonmem/(colorgradient*1024), 0,1,0, 1,1,0, 1,0,0)
			tooltip:AddDoubleLine(format("|cff06DDFA[%d] hidden addons|r |cff0deb11(memory < %d kb)|r:", hidden, 60), formatMem(hiddenmem), nil, nil, nil, r, g, b)
			
			tooltip:AddDoubleLine(" ", "|cffffffff------------|r")
			
			local r, g, b = ColorGradient(addonmem/(colorgradient*1024), 0,1,0, 1,1,0, 1,0,0)
			local memstr = formatMem(addonmem) 
			tooltip:AddDoubleLine(" ", format("|cffC3771ATOTAL USER ADDON|r |cffffffffmemory usage:|r  %s", memstr), nil, nil, nil, r,g,b)
			tooltip:AddDoubleLine(" ", " ")		
				
			local mem = collectgarbage("count")
			local deltamem = mem - prevmem
			prevmem = mem
			local r, g, b = ColorGradient(mem/((colorgradient+20)*1024), 0,1,0, 1,1,0, 1,0,0)
			tooltip:AddDoubleLine("|cffC3771ADefault Blizzard UI|r memory usage:", format("%s", formatMem(mem-addonmem)), nil, nil, nil, r,g,b)
				
			local r, g, b = ColorGradient(mem/((colorgradient+40)*1024), 0,1,0, 1,1,0, 1,0,0)
			tooltip:AddDoubleLine("|cff06DDFATOTAL UI|r memory usage:", format("%s", formatMem(mem)), nil,nil,nil, r,g,b)
				
			tooltip:AddDoubleLine("|cffffffff------------|r", "|cffffffff------------|r")
			
			local r, g, b = ColorGradient(deltamem/15, 0,1,0, 1,1,0, 1,0,0)
			tooltip:AddDoubleLine("|cffc3771aGarbage|r churn", format("%.2f kb/sec", deltamem), nil,nil,nil, r,g,b)
			
			tooltip:AddLine(" ")
			tooltip:AddLine("Click to force |cffc3771agarbage collection|r!")
		end
		LDBLatency.OnClick = function(self)
			local collected, deltamem = 0, 0
			collected = collectgarbage('count')
			collectgarbage("collect")
			UpdateAddOnMemoryUsage()
			deltamem = collected - collectgarbage('count')
					
			print(format("|cffC3771AGarbage|r Collected: |cff06ddfa%s|r", formatMem(deltamem, true)))
		end	
	end
end

--Pet Battle
local LDBPetBattle = LDB:NewDataObject('nioLDBPetBattle', { 
	type = 'launcher',
	icon = [[Interface\Icons\Ability_Hunter_Quickshot]],
	text = 'PetBattle',
})
LDBPetBattle.OnClick = function(self, button)
	if button == 'RightButton' then C_PetBattles.StartPVPMatchmaking()
	else securecall(TogglePetJournal) end
end
LDBPetBattle.OnTooltipShow = function(self)
	self:AddLine('|cff0090ffnioBroker Pet Battles|r')
	self:AddLine("|cff69ccf0Click|cffffd200 to open Pet Journal")
	self:AddLine("|cff69ccf0Right-Click|cffffd200 to start pet battle queue")
end