﻿-- based on bengir_minimap

local cfg = nioConfig.Map
if not cfg.enable then return end
--------------------------------------------------------------------------------------------------------------
-- CONFIG START
--------------------------------------------------------------------------------------------------------------
-- ALT OR SHIFT CLICK to move the minimap

-- Minimap
local map_scale = cfg.map_scale    					-- Scale of the minimap
local map_position = cfg.map_position				-- Initial position. Values = "TOPRIGHT", "TOPLEFT", "BOTTOMRIGHT", "BOTTOMLEFT", "TOP", "BOTTOM", "RIGHT", "LEFT"
local map_offsetX = cfg.map_offsetX					-- X Offset from map_position
local map_offsetY = cfg.map_offsetY					-- Y Offset from map_position

local map_hideInstanceDifficulty = cfg.map_hideInstanceDifficulty		-- Hides / shows the instance difficulty icon on the minimap. Values = true/false

local map_showCoordinates = cfg.map_showCoordinates			-- Shows the X,Y coordinates of your current location. Values = true/false

-- Quest 
-- local quest_parent = UIParent           	-- Defines what frame the anchor position is relative to. Values = "Minimap" or "UIParent". 
-- local quest_anchor = "TOPRIGHT"         	-- Where to anchor on the quest_parent.  Values = "TOPRIGHT", "TOPLEFT", "BOTTOMRIGHT", "BOTTOMLEFT", "TOP", "BOTTOM", "RIGHT", "LEFT"
-- local quest_offsetX = 11			 		-- X Offset
-- local quest_offsetY = -210          	 	-- Y Offset
-- local quest_height = 500                	-- Determines the height of the quest tracker
-- local quest_scale = 0.80            		-- Set the scale of the quest tracker
--------------------------------------------------------------------------------------------------------------
-- CONFIG END
--------------------------------------------------------------------------------------------------------------


--------------------------------------------------------------------------------------------------------------
-- CODING START 
--------------------------------------------------------------------------------------------------------------

Minimap:ClearAllPoints()
Minimap:SetPoint(map_position, UIParent, map_offsetX, map_offsetY)

----------------------------------------------
-- Minimap Options
----------------------------------------------
Minimap:SetMaskTexture('Interface\\AddOns\\nioUI\\Media\\Mask-SQUARE')
Minimap:SetScale(map_scale)
Minimap:SetFrameStrata("BACKGROUND")

Minimap:SetMovable(true)
Minimap:EnableMouse(true)
Minimap:RegisterForDrag("LeftButton","RightButton")
Minimap:SetScript("OnDragStart", function(self) if IsAltKeyDown() or IsShiftKeyDown() then self:StartMoving() end end)
Minimap:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
Minimap:SetUserPlaced(false)
Minimap:SetClampedToScreen(true)


----------------------------------------------
-- Hide buttons of minimap
----------------------------------------------
MinimapBorder:Hide()
MinimapBorderTop:Hide()
MinimapZoomIn:Hide()
MinimapZoomOut:Hide()
MiniMapTracking:Hide()
MiniMapVoiceChatFrame:Hide()
MinimapZoneTextButton:Hide()
GameTimeFrame:Hide()
MinimapNorthTag:SetAlpha(0)
MiniMapWorldMapButton:Hide()


----------------------------------------------
-- Mail Icon
----------------------------------------------
MiniMapMailFrame:ClearAllPoints()
MiniMapMailFrame:SetPoint("TOPRIGHT", Minimap, 2, 2)


----------------------------------------------
--Battlefield Icon
----------------------------------------------
-- MiniMapBattlefieldFrame:ClearAllPoints()
-- MiniMapBattlefieldFrame:SetPoint("BOTTOMLEFT", Minimap, 2, 0)
-- MiniMapBattlefieldFrame:SetFrameLevel(3)
-- MiniMapBattlefieldBorder:Hide()


----------------------------------------------
-- Instance difficulty Icon
----------------------------------------------
if (map_hideInstanceDifficulty == true) then
	MiniMapInstanceDifficulty:ClearAllPoints()
	GuildInstanceDifficulty:ClearAllPoints()
	GuildInstanceDifficulty:Hide()
	MiniMapInstanceDifficulty:Hide()
else
	MiniMapInstanceDifficulty:SetScale(0.7)
	MiniMapInstanceDifficulty:ClearAllPoints()
	MiniMapInstanceDifficulty:SetParent(Minimap)
	MiniMapInstanceDifficulty:SetPoint("TOPRIGHT", Minimap, 1, 1)
	
	GuildInstanceDifficulty:SetScale(0.7)
	GuildInstanceDifficulty:ClearAllPoints()
    GuildInstanceDifficulty:SetParent(Minimap)
    GuildInstanceDifficulty:SetPoint("TOPRIGHT", Minimap, 1, 1)
end


----------------------------------------------
-- LFG Icon
----------------------------------------------
--MiniMapLFGFrame
QueueStatusMinimapButton:ClearAllPoints()
QueueStatusMinimapButton:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 2, 1)
-- LFGSearchStatus:ClearAllPoints()
-- LFGSearchStatus:SetPoint(map_position, QueueStatusMinimapButton, map_position)
-- LFGSearchStatus:SetClampedToScreen(true)
QueueStatusMinimapButtonBorder:SetAlpha(0)
DropDownList1:SetClampedToScreen(true)

----------------------------------------------
-- Time
----------------------------------------------
if not IsAddOnLoaded("Blizzard_TimeManager") then
	LoadAddOn("Blizzard_TimeManager")
end
local clockFrame, clockTime = TimeManagerClockButton:GetRegions()
clockFrame:Hide()
clockTime:Hide()


----------------------------------------------
-- Coordinates
----------------------------------------------
if map_showCoordinates then
	local MapCoordsFrame = CreateFrame('Frame', "MapCoordsFrame", Minimap)
	MapCoordsFrame:SetPoint("TOP", Minimap, "TOP", 0, 15)
	MapCoordsFrame:SetHeight(10)
	MapCoordsFrame:SetWidth(80)
	--We put this here to make sure we draw above addons like gathermate / gatherer who display node icons on the minimap.
	MapCoordsFrame:SetFrameStrata("HIGH")
	local MapCoordsFrameText = MapCoordsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	MapCoordsFrameText:SetPoint("CENTER", MapCoordsFrame, "CENTER", 0, 1)
	MapCoordsFrameText:SetFont(nioConfig.font, 10, "OUTLINE")

	local UpdateInterval = 0.3
	local TimeSinceLastUpdate = 0 
	local x,y

	MapCoordsFrame:SetScript("OnUpdate", function(self, elapsed)	
		TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed
		
		if (TimeSinceLastUpdate > UpdateInterval) then
			x,y = GetPlayerMapPosition("player")
			MapCoordsFrameText:SetFormattedText("%.2f  %.2f", 100*x, 100*y)
			TimeSinceLastUpdate = TimeSinceLastUpdate - UpdateInterval
		end
	end)
end


----------------------------------------------
-- Mouse Wheel
----------------------------------------------
Minimap:EnableMouseWheel(true)
Minimap:SetScript('OnMouseWheel', function(self, delta)
    if delta > 0 then
        Minimap_ZoomIn()
    else
        Minimap_ZoomOut() 
    end
end)

Minimap:SetScript('OnMouseUp', function(self, button)
    if (button == 'RightButton') then
        ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, - (Minimap:GetWidth() * 0.7), -3)
    elseif (button == 'MiddleButton') then
        ToggleCalendar()
    else
        Minimap_OnClick(self)
    end
end)


----------------------------------------------
-- Quest Watch
----------------------------------------------
-- WatchFrame:ClearAllPoints()	
-- WatchFrame.ClearAllPoints = function() end
-- WatchFrame:SetPoint("TOPRIGHT", quest_parent, quest_anchor, quest_offsetX, quest_offsetY)

-- WatchFrame:SetScale(quest_scale)

-- WatchFrame.SetPoint = function() end
-- WatchFrame:SetClampedToScreen(false)
-- WatchFrame:SetHeight(quest_height)


function GetMinimapShape() return 'SQUARE' end
--------------------------------------------------------------------------------------------------------------
-- CODING END 
--------------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------------------------------------------------
-- SLASH COMMANDS START
--------------------------------------------------------------------------------------------------------------
-- SLASH_NIOMINIMAP1, SLASH_NIOMINIMAP2 = '/niomm', '/niominimap'
-- local function bmHandler(msg)
	-- local args = {}
	-- for word in string.gmatch(msg,"[^%s]+") do
		-- table.insert(args,word)
	-- end

	-- if args[1] == 'reset' then
		-- Minimap:ClearAllPoints()
		-- Minimap:SetPoint(map_position, UIParent, map_offsetX, map_offsetY)

	-- elseif args[1] == 'getpos' then
		-- point, relativeTo, relativePoint, xOfs, yOfs = Minimap:GetPoint()
		-- print ("map_position: " .. relativePoint)
		-- print ("map_offsetX: " .. string.format("%d ", xOfs))
		-- print ("map_offsetY: " .. string.format("%d ", yOfs))

	-- elseif args[1] == 'movex' then
		-- point, relativeTo, relativePoint, xOfs, yOfs = Minimap:GetPoint()
		-- Minimap:ClearAllPoints()
		-- Minimap:SetPoint(map_position, UIParent, xOfs+args[2], yOfs)

	-- elseif args[1] == 'movey' then
		-- point, relativeTo, relativePoint, xOfs, yOfs = Minimap:GetPoint()
		-- Minimap:ClearAllPoints()
		-- Minimap:SetPoint(map_position, UIParent, xOfs, yOfs+args[2])

	-- else 
		-- print (" ")
		-- print ("NioMinimap options:")
		-- print (" ")
		-- print ("'reset' to reset the minimap to it's original position.")
		-- print ("'getpos' to get the current x and y offset of the minimap.")
		-- print ("'movex [number]' move the minimap on the x axis by [number] of pixels.")
		-- print ("'movey [number]' move the minimap on the y axis by [number] of pixels.")

	-- end
-- end
-- SlashCmdList["NIOMINIMAP"] = bmHandler
--------------------------------------------------------------------------------------------------------------
-- SLASH COMMANDS END
--------------------------------------------------------------------------------------------------------------
