-- based on Select

local ssplit = function(str,delim)
	local t = {}
	local fpat = "(.-)" .. delim
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	while s do
		if s ~= 1 or cap ~= "" then
			table.insert(t,cap)
		end
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	if last_end <= #str then
		cap = str:sub(last_end)
		table.insert(t, cap)
	end
	return t
end

nioMount = {
	macroCurrentlyOpen = nil, -- id of macro that summoned the menu
	buttonCurrentlyOpen = nil, -- button that contains the macro

	Menu = {},
}

CreateFrame("GameTooltip","nioMountTooltipScan",UIParent,"GameTooltipTemplate")

function nioMount.SlashHandler(msg)
	local focus = GetMouseFocus()
	local action = focus:GetAttribute("action") or focus.action
	if action and tonumber(action) then
		local actionType,macroIndex = GetActionInfo(action)
		if actionType=="macro" then
			if strfind(GetMacroBody(macroIndex) or "",msg) then
				-- user's mouse is over the macro that called this
				nioMount.CloseMenu()
				nioMount.PopulateMenu(focus)
				nioMount.macroCurrentlyOpen = macroIndex
				nioMount.buttonCurrentlyOpen = focus
			end
		end
	end
end


local function nioMountHandler(msg)
	local btn = GetRunningMacroButton() 
	if btn == "RightButton" then return end	
	local fly, ground = string.split(",", msg, 2)
	
	if UnitInVehicle("player") and CanExitVehicle() then 
		VehicleExit() 
	elseif IsMounted() then
		Dismount()
	elseif IsShiftKeyDown() and select(2, UnitClass("player")) == "SHAMAN" then
		CastSpellByID(2645)
	elseif IsModifierKeyDown() or SecureCmdOptionParse("[flyable] t; f") ~= "t" then
		CastSpellByName(ground)
	else
		CastSpellByName(fly)
	end
end

SlashCmdList["NIOMOUNTINTERN"] = nioMountHandler
SLASH_NIOMOUNTINTERN1 = "/nioselec"


function nioMount.AddToMenu(item)
	local found
	for i=1,#(nioMount.Menu) do
		if nioMount.Menu[i]==item then
			found = 1
			break
		end
	end
	if not found then
		table.insert(nioMount.Menu,item)
	end
end

function nioMount.PopulateMenu(focus)
	for i in pairs(nioMount.Menu) do
		nioMount.Menu[i] = nil
	end
	
	for i=1,GetNumCompanions("MOUNT") do
		nioMount.AddToMenu((select(2,GetCompanionInfo("MOUNT",i))))
	end
	
	nioMount.BuildMenu(focus, "companion", "MOUNT")
end

SlashCmdList["NIOMOUNT"] = nioMount.SlashHandler
SLASH_NIOMOUNT1 = "/niomount"

function nioMount.GetMenuDocking(button)
	local buttonName = button:GetName()
	if not buttonName then
		return
	end
	local orient = "HORIZONTAL"
	local buttonStub,buttonNumber = string.match(buttonName,"(.-)(%d+)")
	buttonNumber = tonumber(buttonNumber)
	if buttonNumber then
		local function near(v1,v2)
			if v1 and v2 then return math.abs(v2-v1)<16 end
		end
		local candidate1 = _G[buttonStub..(buttonNumber-1)]
		local candidate2 = _G[buttonStub..(buttonNumber+1)]
		if (candidate1 and near(candidate1:GetLeft(),button:GetLeft())) or
			(candidate2 and near(candidate2:GetLeft(),button:GetLeft())) then
			orient = "VERTICAL"
		end
	end

	local buttonTop,buttonLeft = button:GetTop(),button:GetLeft()
	local midHeight,midWidth = UIParent:GetHeight()/2,UIParent:GetWidth()/2

	if buttonTop>midHeight and orient=="HORIZONTAL" then -- on a horizontal bar in top half of screen
		if buttonLeft<midWidth then
			return "TOPLEFT", "BOTTOMLEFT", 36,0, 0,-36
		else
			return "TOPRIGHT", "BOTTOMRIGHT", -36,0, 0,-36
		end
	elseif buttonLeft<=midWidth and orient=="VERTICAL" then -- on a vertical bar in left half of screen
		if buttonTop>midHeight then
			return "TOPLEFT", "TOPRIGHT", 0,-40, 40,0
		else
			return "BOTTOMLEFT", "BOTTOMRIGHT", 0,40, 40,0
		end
	elseif buttonLeft>midWidth and orient=="VERTICAL" then -- on a vertical bar in right half of screen
		if buttonTop>midHeight then
			return "TOPRIGHT", "TOPLEFT", 0,-36, -36,0
		else
			return "BOTTOMRIGHT", "BOTTOMLEFT", 0,36, -36,0
		end
	else -- all other conditions, assume button is on a horizontal bar in bottom half of screen
		if buttonLeft>midWidth then
			return "BOTTOMRIGHT", "TOPRIGHT", -36,0, 0,36
		else
			return "BOTTOMLEFT", "TOPLEFT", 36,0, 0,36
		end
	end
end

function nioMount.OnTooltip(owner)
	GameTooltip:SetOwner(owner,"ANCHOR_LEFT")
	GameTooltip[owner.tooltipMethod](GameTooltip,owner.tooltipParam,"spell")
	GameTooltip:Show()
end

function nioMount.MenuOnUpdate(self,elapsed)
	self.timer = self.timer + elapsed
	if self.timer > .5 then
		self.timer = 0
		local f = GetMouseFocus()
		if f==nioMount.buttonCurrentlyOpen or string.match(f:GetName() or "","nioMountMenuButton") then
			return
		else
			nioMount.CloseMenu()
		end
	end
end

function nioMount.CreateButton(idx,entryType,entryName,entryCriterea)
	local buttonName = "nioMountMenuButton"..idx
	local button = _G[buttonName]
	if not button then
		button = _G[buttonName] or CreateFrame("Button",buttonName,UIParent,"ActionButtonTemplate")
		button:SetFrameStrata("DIALOG")
		button:SetScript("OnClick",function() nioMount.MenuSelected(button) end)
		button:SetScript("OnEnter",function() nioMount.OnTooltip(button) end)
		button:SetScript("OnLeave",function() GameTooltip:Hide() end)
		if button==nioMountMenuButton1 then
			button.timer = 0
			button:SetScript("OnUpdate",nioMount.MenuOnUpdate)
		end
	end
	_G[buttonName.."Count"]:SetText("")
	button.entryName = entryName
	button.entryType = entryType
	button.entryCriterea = entryCriterea

	local _,_,spellId,texture = GetCompanionInfo("MOUNT",idx)
	_G[buttonName.."Icon"]:SetTexture(texture)
	button.tooltipMethod = "SetHyperlink"
	button.tooltipParam = "spell:"..spellId
	button.compspell = GetSpellInfo(spellId)

	button:ClearAllPoints()
	return button
end

function nioMount.BuildMenu(button,menuType,menuCriterea)
	local relativeFrom,relativeTo,addx,addy,addxwrap,addywrap = nioMount.GetMenuDocking(button)
	local xstart,ystart = 0,0 -- these values change only when a row wraps
	local xpos,ypos = xstart,ystart -- these values change for every new button
	local menuButton
	local wrapat = 1
	local wraplimit = 1
	local numEntries = #(nioMount.Menu)
	if numEntries > 25 then
		wraplimit = 5
	elseif numEntries > 18 then
		wraplimit = 4
	elseif numEntries > 9 then
		wraplimit = 3
	elseif numEntries > 4 then
		wraplimit = 2
	end

	for i=1,#(nioMount.Menu) do
		menuButton = nioMount.CreateButton(i,menuType,nioMount.Menu[i],menuCriterea)
		menuButton:SetPoint(relativeFrom,button,relativeTo,xpos,ypos)
		menuButton:Show()
		xpos=xpos+addx
		ypos=ypos+addy
		wrapat = wrapat + 1
		if wrapat>wraplimit then
			xstart=xstart+addxwrap
			ystart=ystart+addywrap
			xpos = xstart
			ypos = ystart
			wrapat = 1
		end
	end
end

function nioMount.CloseMenu()
	local idx = 1
	while _G["nioMountMenuButton"..idx] do
		_G["nioMountMenuButton"..idx]:Hide()
		idx = idx + 1
	end
end

function nioMount.MenuSelected(button)
	nioMount.CloseMenu()
	if not InCombatLockdown() then
		local newBody
		local oldName,_,oldBody,isLocal = GetMacroInfo(nioMount.macroCurrentlyOpen)
		
		local mountfly, mountground
		local pos,pos2
		if IsShiftKeyDown() then
			mountground = button.compspell
			pos = string.find(oldBody, "/nioselec")
			pos2 = string.find(oldBody, ",",pos)
			if pos == nil then mountfly = mountground
			else
				mountfly = string.sub(oldBody, pos+10,  pos2-1)
				if mountfly == "" then mountfly = mountground end
			end
		else
			mountfly = button.compspell
			pos = string.find(oldBody, "/nioselec")
			pos2 = string.find(oldBody, ",",pos)
			if pos == nil then mountground = mountfly
			else
				pos = string.find(oldBody, "/stopmacro")
				mountground = string.sub(oldBody, pos2+1, pos-2)
				if mountground == "" then mountground = mountfly end
			end			
		end
		
		--shamans can use ghostwolf in combat
		local combat = ""
		if select(2, UnitClass("player")) == "SHAMAN" then
			combat = "\n/cast [combat] "..GetSpellInfo(2645)
		end
		
		local mounttemplate = "#showtooltip [mod] [noflyable] %s;%s%s\n/nioselec %s,%s\n/stopmacro [nobtn:2]\n/niomount"
		newBody = string.format(mounttemplate,
			mountground,
			mountfly,
			combat,
			mountfly,
			mountground -- name of item or spell
			--button.entryType, -- type in /select type:criterea
			--button.entryCriterea -- criterea in /select type:criterea
		)
			
		EditMacro(nioMount.macroCurrentlyOpen,oldName,"INV_MISC_QUESTIONMARK",newBody,isLocal)
	else
		print("You can't /niomount in combat, sorry.")
	end
end
