--------------------------------------
-- NIONAMEPLATES
--------------------------------------
local cfg = nioConfig.Nameplates
if not cfg.enable then return end

local nioNameplates = CreateFrame("Frame")
local total = -1
local InCombat = false
local DPSMode

--------------------------------------
-- SETTINGS
--------------------------------------
local ThreatLow = { r = 0.0, g = 0.7, b = 0.0 }
local ThreatMed = { r = 0.7, g = 0.7, b = 0.0 }
local ThreatHigh = { r = 0.8, g = 0.0, b = 0.0 }

--------------------------------------
-- UPDATE THREAT
--------------------------------------

local function UpdateThreat(NamePlate, elapsed)	
	NamePlate.elapsed = NamePlate.elapsed + elapsed	
	if NamePlate.elapsed < 0.2 then return end
	NamePlate.elapsed = 0
		
	if InCombat then NamePlate.Threat:Show()
	elseif not InCombat and NamePlate.Threat:IsShown() then NamePlate.Threat:Hide() return 
	else return end 
	
	local rt, gt = NamePlate.BlizzThreat:GetVertexColor()
	
	if not NamePlate.BlizzThreat:IsShown() then 
		if DPSMode then NamePlate.Healthbar:SetStatusBarColor(ThreatLow.r, ThreatLow.g, ThreatLow.b, 1)
		else NamePlate.Healthbar:SetStatusBarColor(ThreatHigh.r, ThreatHigh.g, ThreatHigh.b, 1) end
	elseif rt > 0 then
		if gt > 0 then NamePlate.Healthbar:SetStatusBarColor(ThreatMed.r, ThreatMed.g, ThreatMed.b, 1)
		elseif DPSMode then NamePlate.Healthbar:SetStatusBarColor(ThreatHigh.r, ThreatHigh.g, ThreatHigh.b, 1)
		else NamePlate.Healthbar:SetStatusBarColor(ThreatLow.r, ThreatLow.g, ThreatLow.b, 1) end
	end
end

--------------------------------------
-- NAMEPLATE FUNCTIONS
--------------------------------------

local function InitPlate(NamePlate)
	NamePlate.Healthbar = NamePlate:GetChildren()
	
	--check if it is a player, no need to do threat on them
	local r, g, b = NamePlate.Healthbar:GetStatusBarColor()
	if (b > 0 and g > 0) then return end
	
	--else make the threat overlay
	NamePlate.BlizzThreat = NamePlate:GetRegions()
    
    NamePlate.Threat = NamePlate:CreateTexture(nil, "Texture")
    NamePlate.Threat:SetPoint("TOPRIGHT", NamePlate.Healthbar, 1, 2)
    NamePlate.Threat:SetPoint("BOTTOMLEFT", NamePlate.Healthbar, -1, -2)
	
	-- Initialization Stuff
	NamePlate.Mod = true
	NamePlate.BlizzThreat:SetTexture(0, 0, 0, 0)
	NamePlate.elapsed = 1
	UpdateThreat(NamePlate, 1)
	
	NamePlate:SetScript("OnUpdate", UpdateThreat)
end

local lastupdate = 0
local function GetNameplates(frame, elapsed)
	lastupdate = lastupdate + elapsed
    if lastupdate < 0.2 then return end
	lastupdate = 0
		
    local current = select("#", WorldFrame:GetChildren())
    if (current ~= total) then
		local NamePlate
    	total = current
	  	for i = 1, current do
			NamePlate = select(i, WorldFrame:GetChildren())
			if (not NamePlate.Mod) and NamePlate:GetName() and NamePlate:GetName():match("NamePlate") then
		  		InitPlate(NamePlate)
		  	end
		end
    end
end

--------------------------------------
-- APPLY CHANGES
--------------------------------------
local function DoChanges()
	nioNameplates:SetScript("OnUpdate", nil)
	nioNameplates:UnregisterEvent("PLAYER_ENTERING_WORLD")
	nioNameplates:UnregisterEvent("PLAYER_REGEN_DISABLED")
	nioNameplates:UnregisterEvent("PLAYER_REGEN_ENABLED")		
			
	local SpecID, role = GetSpecialization()
	if SpecID == nil then role = "NONE"
	else role = GetSpecializationRole(SpecID)end	
	
	if role == "TANK" or not cfg.OnlyForTanks then
		nioNameplates:RegisterEvent("PLAYER_REGEN_DISABLED")
		nioNameplates:RegisterEvent("PLAYER_REGEN_ENABLED")
		nioNameplates:SetScript("OnUpdate", GetNameplates)			
		if role == "TANK" then DPSMode = false
		else DPSMode = true end
	end
end

--------------------------------------
-- INITIALIZATION
--------------------------------------
nioNameplates:SetScript("OnEvent", function(self, event)
	if (event == "PLAYER_REGEN_DISABLED") then InCombat = true
	elseif (event == "PLAYER_REGEN_ENABLED") then InCombat = false
	elseif  event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TALENT_UPDATE" or event == "CHARACTER_POINTS_CHANGED" then	
		DoChanges()
	end
	collectgarbage()
end) 
nioNameplates:RegisterEvent("PLAYER_ENTERING_WORLD")
nioNameplates:RegisterEvent("PLAYER_TALENT_UPDATE")
nioNameplates:RegisterEvent("CHARACTER_POINTS_CHANGED")