-- Register the media with shared media
local LSM = LibStub("LibSharedMedia-3.0")
LSM:Register("font", "Electrofied",	[[Interface\Addons\nioUI\fonts\electr.ttf]])
LSM:Register("font", "nioMonospaced",	[[Interface\Addons\nioUI\fonts\ProFontWindows.ttf]])
LSM:Register("border", "nioUI_Border", [[Interface\AddOns\nioUI\Media\UI-DialogBox-Border.blp]])
LSM:Register("border", "nioUI_WhiteBorder", [[Interface\AddOns\nioUI\Media\UI-DialogBox-Border-White.tga]])
LSM:Register("statusbar", "nioUI_Bar",	[[Interface\AddOns\nioUI\Media\Minimalist.tga]])

-- show the panels
lpanels:ApplyLayout(nil, "nioUI")

---------------------------------
-- SLASH COMMANDS START
---------------------------------
SLASH_NIOUI1, SLASH_NIOUI2, SLASH_NIOUI3 = '/nioui', '/nioUI', '/nio'
local function nioHandler(msg)
	local args = {}
	for word in string.gmatch(msg,"[^%s]+") do
		table.insert(args,word)
	end

	if args[1] == 'install' then
		nioUIInstall = false
		bb08df87101dd7f2161e5b77cf750f753c58ef1b = {}
		SlashCmdList.RESETBARS()
		ReloadUI()
	elseif args[1] == 'hb' or args[1] == 'bind' then
		SlashCmdList.MOUSEOVERBIND()
	elseif args[1] == 'omf' then
		SlashCmdList.OMF()	
	elseif args[1] == 'mb' then
		SlashCmdList.MOVEFRAMES()	
	elseif args[1] == 'rb' then
		SlashCmdList.RESETBARS()	
	elseif args[1] == 'skin' then
		SlashCmdList.SKIN()	
	elseif args[1] == 'help' then
		print (" ")
		print ("nioUI options:")
		print ("  'install' - to reinstall the UI and reskin supported addons.")
		print ("  'hb' - to enter the hover bind mode.")
		print ("  'omf' - to move and scale the unitframes.")
		print ("  'mb' - to move the actionbars.")
	else 
		InterfaceOptionsFrame_OpenToCategory("nioUI")
	end
end
SlashCmdList["NIOUI"] = nioHandler
----------------------------------
-- SLASH COMMANDS END
----------------------------------

-- Do this stuff on player entering world (after addons are loaded)
local nioUI = CreateFrame"Frame"
nioUI:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		-- Hide Blizzard Raidframes
		CompactRaidFrameManager:UnregisterAllEvents()
		CompactRaidFrameManager:Hide()
		CompactRaidFrameContainer:UnregisterAllEvents()
		CompactRaidFrameContainer:Hide()
		
		-- Hide some Minimap Buttons
		local icon = LibStub("LibDBIcon-1.0", true)
		if IsAddOnLoaded("DBM-Core") then DBM:HideMinimapButton() end
		if IsAddOnLoaded("MarkAssistant") then MA_Form:Hide() end
		if IsAddOnLoaded("Skada") then icon:Hide("Skada") end
		if IsAddOnLoaded("BugSack") then icon:Hide("BugSack") end
		if IsAddOnLoaded("LoggerHead") then icon:Hide("LoggerHead") end	
		
		-- Install the UI
		if not nioUIInstall then 
			nioUIInstall = true
			
			SlashCmdList.RESETBARS()
			SlashCmdList.SKIN()
			
			-- Enable Actionbars
			if nioConfig.Actionbars.enable and nioConfig.Actionbars.autoenable then 
				SHOW_MULTI_ACTIONBAR_1 = 1
				SHOW_MULTI_ACTIONBAR_2 = 1
				SHOW_MULTI_ACTIONBAR_3 = 1
				SHOW_MULTI_ACTIONBAR_4 = 1
				MultiActionBar_Update()
				SetActionBarToggles(1, 1, 1, 1)
			end
		
			StaticPopupDialogs["NIOUI_INSTALL"] = {
				text = "Welcome to nioUI! \nClick on the button and your new UI will be ready! \n (type /nioui to open the configuration)",
				button1 = OKAY,
				OnAccept = function()
				  ReloadUI()
				end,
				timeout = 0,
				whileDead = true,
			}	
			StaticPopup_Show ("NIOUI_INSTALL")
		end
		
		nioUI:UnregisterEvent("PLAYER_ENTERING_WORLD")
		nioUI = nil
	end	
end)
nioUI:RegisterEvent("PLAYER_ENTERING_WORLD")

-- Make the Interface Menu moveable
local function makeMovable(frame)
	local mover = CreateFrame("Frame", frame:GetName() .. "Mover", frame)
	mover:EnableMouse(true)
	mover:SetPoint("TOP", frame, "TOP", 0, 10)
	mover:SetWidth(160)
	mover:SetHeight(40)
	mover:SetScript("OnMouseDown", function(self)
		self:GetParent():StartMoving()
	end)
	mover:SetScript("OnMouseUp", function(self)
		self:GetParent():StopMovingOrSizing()
	end)
	frame:SetMovable(true)
end
makeMovable(InterfaceOptionsFrame)

-- Positioning the LootRollFrame
local lcfg = nioConfig.LootRoll
local function LoadPositionHook()
	local frame = _G["GroupLootContainer"]
	if frame then
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMLEFT", UIParent, lcfg.anchor, lcfg.x_off, lcfg.y_off)
		frame:SetParent(UIParent)
		frame:SetScale(lcfg.scale)
	end
	
	for i=1, NUM_GROUP_LOOT_FRAMES do
		frame = _G["GroupLootFrame" .. i]
		if i == 1 then
			if frame then
				frame:ClearAllPoints()
				frame:SetPoint("BOTTOMLEFT", UIParent, lcfg.anchor, lcfg.x_off, lcfg.y_off)
				frame:SetParent(UIParent)
				frame:SetScale(lcfg.scale)
			end
		else
			if frame then
				frame:ClearAllPoints()
				frame:SetPoint("BOTTOM", "GroupLootFrame" .. (i-1), lcfg.grow, 0, 7)
				frame:SetParent(UIParent)
				frame:SetScale(lcfg.scale)
			end
		end
	end	
end
hooksecurefunc("GroupLootContainer_OnLoad", LoadPositionHook)
hooksecurefunc("GroupLootContainer_RemoveFrame", LoadPositionHook)
hooksecurefunc("GroupLootFrame_OnShow", LoadPositionHook)
hooksecurefunc("GroupLootFrame_OpenNewFrame", LoadPositionHook)
hooksecurefunc("GroupLootFrame_OnEvent", LoadPositionHook)
hooksecurefunc("GroupLootContainer_Update", LoadPositionHook)
hooksecurefunc("AlertFrame_FixAnchors", LoadPositionHook)


-----------------------------
-- Get Player Class and Roles
-----------------------------
nioConfig.Class = select(2, UnitClass("player"))
local PlayerRole = CreateFrame("Frame")
nioRole = nioRole or {}
local function GetRole(self, event, unit)
	if  event == "PLAYER_ENTERING_WORLD" or
		event == "PLAYER_TALENT_UPDATE" or
		event == "CHARACTER_POINTS_CHANGED" then
		for i=1, GetNumSpecGroups() do
			local id = GetSpecialization(false, false, i)
			if id == nil then role = "NONE"
			else
				local role = GetSpecializationRole(id)
				nioRole[i] = role
			end
		end
	end
	
	local ActiveTree = GetActiveSpecGroup()
	if ActiveTree == 1 then nioRole[3] = nioRole[1]
	else nioRole[3] = nioRole[2] end
	
	if nioRole[3] == "TANK" then nioConfig.NameplatesTankmode = false
	else nioConfig.NameplatesTankmode = true end
	
end	
PlayerRole:RegisterEvent("PLAYER_ENTERING_WORLD")
PlayerRole:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
PlayerRole:RegisterEvent("PLAYER_TALENT_UPDATE")
PlayerRole:RegisterEvent("CHARACTER_POINTS_CHANGED")
PlayerRole:SetScript("OnEvent", GetRole)

-----------------------------
-- Set Role automatic
-----------------------------
local RoleSetter = CreateFrame("Frame")
local function SetRole(isPoll)
	if not nioConfig.Misc.SetRoleAutomatic then return end
	if (nioRole[3] == nil or (not isPoll and GetNumGroupMembers() <= 0)) then return end
	
	UnitSetRole("player", nioRole[3])
end

local function SetRole_OnEvent(self, event) 
  if (event == "ROLE_POLL_BEGIN") then
	SetRole(true)
	StaticPopupSpecial_Hide(RolePollPopup)
  elseif (event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED") then
	SetRole(false)
  end  
end
RoleSetter:SetScript("OnEvent", SetRole_OnEvent)
RoleSetter:RegisterEvent("PLAYER_TALENT_UPDATE")
RoleSetter:RegisterEvent("RAID_ROSTER_UPDATE")
RoleSetter:RegisterEvent("ROLE_POLL_BEGIN")
RoleSetter:RegisterEvent("PARTY_MEMBERS_CHANGED")


--Autorepair and sell grey
if nioConfig.Misc.autorepair or nioConfig.Misc.sellgray then
	local MerchantActions = CreateFrame("Frame")
	local function MerchantActions_OnEvent(self, event) 
		if nioConfig.Misc.autorepair then
			local cost, canRepair = GetRepairAllCost()
			if (canRepair==1) then
				if nioConfig.Misc.guildfunds then
					if (CanGuildBankRepair()==1) then
						RepairAllItems(1)
						print("Repaired all items with guild funds")
					else
						RepairAllItems()
						print("Repaired all items, guild funds unavailable")
					end
				else
					if (cost<=GetMoney()) then
						RepairAllItems()
						print("Repaired all items")
					elseif (cost>GetMoney()) then
						print("You don't have enough money to repair your armor!")
					end
				end
			end		
		end			
		if nioConfig.Misc.sellgray then
			local sold = false
			for bagNum = 0,4 do
				for slotNum = 1,GetContainerNumSlots(bagNum) do
					local item = GetContainerItemLink(bagNum, slotNum)
					if (item) then
						local _,_,qual = GetItemInfo(item)
						if (qual==0) then
							PickupContainerItem(bagNum, slotNum)
							PickupMerchantItem()
							sold = true
						end
					end
				end
			end
			if (sold==true) then
				print("Some gray quality items have been automatically sold.")
			end
		end		
	end
	MerchantActions:SetScript("OnEvent", MerchantActions_OnEvent)
	MerchantActions:RegisterEvent("MERCHANT_SHOW")	
end