--CREDITS: Xrystal for nUI support code, seebs for line spacing code
--------------------------------------------------------------------
local cfg = nioConfig.Quest
if not cfg.enable then return end

local skin = "Interface\\AddOns\\rActionButtonStyler\\media\\gloss.tga"		--the normal button skin texture
local hover = "Interface\\AddOns\\rActionButtonStyler\\media\\hover.tga"	--the texture when hovering your mouse over the button
local pushed = "Interface\\AddOns\\rActionButtonStyler\\media\\pushed.tga"	--the texture when pushing the button

local WFContainer = CreateFrame("Frame", "WFContainer", UIParent)
local db, SetSize, MoveWatchFrame, Appearance, FontChanges, line	--, WFWW_BF, itembtn
local nextline = 1
local nextbutton = 1
local GameTooltip = GameTooltip
local WatchFrameCollapseExpandButton = WatchFrameCollapseExpandButton
local positions = {"TOP", "TOPRIGHT", "TOPLEFT",
					"BOTTOM", "BOTTOMRIGHT", "BOTTOMLEFT",
					"RIGHT", "LEFT", "CENTER"}
local fontFlags = {"None", "Outline", "Thick Outline", "Monochrome"}
local wideFrame = GetCVar("watchFrameWidth")
local _,class = UnitClass("player")
local classColor
-- local WFWWDB
-- local WFWWPCDB


local defaults = {
	["classBorder"] = false,
	["fontSize"] = cfg.font_size,
	["xOffset"] = cfg.x_off,
	["border"] = nioConfig.Border,
	["yOffset"] = cfg.y_off,
	["anchorPoint"] = cfg.anchor,
	["borderColor"] = {
		["r"] = 1,
		["g"] = 1,
		["b"] = 1,
	},
	["bg"] = "Interface\\Tooltips\\UI-Tooltip-Background",
	["borderAlpha"] = 0.8,
	["questSpacing"] = cfg.questSpacing,
	["keybind"] = "",
	["disableShadow"] = false,
	["fontFlag"] = "None",
	["borderWidth"] = 24,
	["instance"] = cfg.instance,
	["font"] = "Friz Quadrata TT",
	["lineSpacing"] = cfg.lineSpacing,
	["hideFrame"] = true,
	["height"] = cfg.height,
	["scale"] = cfg.scale,
	["bgColor"] = {
		["a"] = nioConfig.bg.a,
		["r"] = nioConfig.bg.r,
		["g"] = nioConfig.bg.g,
		["b"] = nioConfig.bg.b,
	},
	["inset"] = 7,
}

function MoveWatchFrame()
	WFContainer:ClearAllPoints()
	WFContainer:SetPoint(db.anchorPoint, UIParent, db.anchorPoint, db.xOffset, db.yOffset)
end

function SetSize()
	WFContainer:SetHeight(db.height)
	if wideFrame == "1" then
		WFContainer:SetWidth(350)
	else
		WFContainer:SetWidth(250)
	end
	WFContainer:SetScale(db.scale)
end

function Appearance()
	WFContainer:SetBackdrop({
			bgFile = db.bg,
			edgeFile = db.border,
			edgeSize = db.borderWidth,
			insets = {left=db.inset,right=db.inset,top=db.inset,bottom=db.inset}
		})
	WFContainer:SetBackdropColor(db.bgColor.r,db.bgColor.g,db.bgColor.b,db.bgColor.a)
	if db.classBorder then
		WFContainer:SetBackdropBorderColor(classColor.r,classColor.g,classColor.b,db.borderAlpha)
	else
		WFContainer:SetBackdropBorderColor(db.borderColor.r,db.borderColor.g,db.borderColor.b,db.borderAlpha)
	end
end

function FontChanges()
	local font = db.font
	WatchFrameTitle:SetFont(font, db.fontSize, db.fontFlag)
	for i = nextline, 50 do
		line = _G["WatchFrameLine"..i]
		if line then
			line.text:SetFont(font, db.fontSize, db.fontFlag)
			line.dash:SetFont(font, db.fontSize, db.fontFlag)
			if db.disableShadow then
				line.text:SetShadowColor(0,0,0,0)
				line.dash:SetShadowColor(0,0,0,0)
			else
				line.text:SetShadowColor(0,0,0,1)
				line.dash:SetShadowColor(0,0,0,1)
			end
		else
			nextline = i	--so we only have to change new lines during the hook
			break
		end
	end
end

local function SkinButtons()
	if IsAddOnLoaded("rActionButtonStyler") then
		local colorR,colorG,colorB = ActionButton1NormalTexture:GetVertexColor()
		for i = nextbutton, 15 do
			itembtn = _G["WatchFrameItem"..i]
			if itembtn then
				itembtn:SetHighlightTexture(hover)
				itembtn:SetPushedTexture(pushed)
				border = _G[itembtn:GetName().."NormalTexture"]
				border:SetAllPoints()
				border:SetTexture(skin)
				border:SetVertexColor(colorR, colorG, colorB)
			else
				nextbutton = i
				break
			end
		end
	end
end

local function SetUpContainer()	
	MoveWatchFrame()
	SetSize()
	Appearance()
	SkinButtons()
	WATCHFRAME_QUEST_OFFSET = db.questSpacing
	WATCHFRAME_LINEHEIGHT = db.fontSize + db.lineSpacing
	
	hooksecurefunc("WatchFrame_Update", function()
		FontChanges()
		SkinButtons()

		if db.hideFrame then
			if GetNumTrackedAchievements() == 0 and GetNumQuestWatches() == 0 then
				WFContainer:SetAlpha(0)
				WatchFrameCollapseExpandButton:EnableMouse(false)
			else
				WFContainer:SetAlpha(1)
				WatchFrameCollapseExpandButton:EnableMouse(true)
			end
		end
	end)
	
	WatchFrame_Update()
	
	hooksecurefunc("WatchFrameLinkButtonTemplate_Highlight", function(self, enter)
		if enter then
			local id = self.index
			local watchType = self.type
			if watchType == "QUEST" then
				GameTooltip:SetOwner(self)
				GameTooltip:SetHyperlink(GetQuestLink(GetQuestIndexForWatch(id)))
				GameTooltip:Show()
			elseif watchType == "ACHIEVEMENT" then
				GameTooltip:SetOwner(self)
				GameTooltip:SetHyperlink(GetAchievementLink(id))
				GameTooltip:Show()			
			end
		else
			GameTooltip:Hide()
		end
	end)
	
	WatchFrame:SetParent(WFContainer)
	WatchFrame:SetClampedToScreen(false)
	WatchFrame:ClearAllPoints()
	WatchFrame.ClearAllPoints = function() end
	WatchFrame:SetPoint("TOPLEFT", 32,-2.5)
	WatchFrame:SetPoint("BOTTOMRIGHT", 4,0)
	WatchFrame.SetPoint = function() end

	WatchFrameTitle:SetParent(WFContainer)
	WatchFrameCollapseExpandButton:SetParent(WFContainer)
	WatchFrameCollapseExpandButton:Raise()	--let us click it even if header tex is too big
	WatchFrameTitle:Show()
	WatchFrameCollapseExpandButton:Show()
	WatchFrameTitle.Hide = function() end
	WatchFrameCollapseExpandButton.Hide = function() end
	WatchFrameCollapseExpandButton.Disable = function() end
	
	if WFWWPCDB.collapsed then	--remember if watchframe was collapsed on last logout
		WatchFrame_CollapseExpandButton_OnClick()	--default function called by OnClick
		WFContainer:SetHeight(30)
		WFContainer:SetWidth(150)
	end
	
	WatchFrameCollapseExpandButton:RegisterForClicks("AnyDown")
	WatchFrameCollapseExpandButton:SetScript("OnClick", function(self,btn)	--redo this to allow for modified clicks
		if IsShiftKeyDown() then
			ToggleAchievementFrame()
		elseif IsAltKeyDown() then
			InterfaceOptionsFrame_OpenToCategory("Who Framed Watcher Wabbit?")
		elseif btn == "RightButton" then
			ToggleFrame(QuestLogFrame)
		else
			WatchFrame_CollapseExpandButton_OnClick()	--default function called by OnClick
			if WatchFrame.userCollapsed then	--my original hook
				WFContainer:SetHeight(30)
				WFContainer:SetWidth(150)
				WFWWPCDB.collapsed = true
			else
				WFContainer:SetHeight(db.height)
				if wideFrame == "1" then
					WFContainer:SetWidth(350)
				else
					WFContainer:SetWidth(250)
				end
				WFWWPCDB.collapsed = false
			end
		end
	end)
	WatchFrameCollapseExpandButton:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_TOP")
			GameTooltip:AddLine("Who Framed Watcher Wabbit?",1,1,1)
			if WatchFrame.collapsed then
				GameTooltip:AddLine("Left-click to expand the objectives tracker.")
			else
				GameTooltip:AddLine("Left-click to minimize the objectives tracker.")
			end
			GameTooltip:AddLine("Right-click to toggle the quest log.")
			GameTooltip:AddLine("Shift-click to toggle the achievements window.")
			GameTooltip:Show()
		end)
	WatchFrameCollapseExpandButton:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

local function SetUpDB()
	if WFWWPCDB.charSpec then
		--set defaults if new charSpec DB
		for k,v in pairs(defaults) do
			if type(WFWWPCDB[k]) == "nil" then
				WFWWPCDB[k] = v
			end
		end
		db = WFWWPCDB
	else
		db = WFWWDB
	end
end

local function Initialize()
	WFWWDB = WFWWDB or {}
	WFWWPCDB = WFWWPCDB or {}
		if WFWWPCDB.charSpec == nil then
			WFWWPCDB.charSpec = false
		end
		if WFWWPCDB.collapsed == nil then
			WFWWPCDB.collapsed = false
		end
	for k,v in pairs(defaults) do
	    if type(WFWWDB[k]) == "nil" then
	        WFWWDB[k] = v
	    end
	end
	SetUpDB()
	
	classColor = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
	if CUSTOM_CLASS_COLORS then
		CUSTOM_CLASS_COLORS:RegisterCallback(function()
				if db.classBorder then
					Appearance()
				end
			end)
	end
	
	if not db.instance then
		WFContainer:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
	WFContainer:RegisterEvent("QUEST_WATCH_UPDATE")
	WFContainer:RegisterEvent("QUEST_AUTOCOMPLETE")
	WFContainer:RegisterEvent("CVAR_UPDATE")
	WFContainer:SetScript("OnEvent", function(_,event,...)
			if event == "PLAYER_ENTERING_WORLD" then
				if IsInInstance() and not WFWWPCDB.collapsed then
				--if you go in the instance and the WF isn't collapsed...
				WatchFrameCollapseExpandButton:Click()	--collapse
				end
			elseif event == "CVAR_UPDATE" then
				local cvar,value = ...
				if cvar == "WATCH_FRAME_WIDTH_TEXT" then
					if not WatchFrame.userCollapsed then
						if value == "1" then
							WFContainer:SetWidth(350)
						else
							WFContainer:SetWidth(250)
						end
					end
					wideFrame = value
				end
			else
				local index = ...
				if event == "QUEST_AUTOCOMPLETE" or index == 0 then
					if WFWWPCDB.collapsed then
						WFContainer:SetHeight(db.height)
						if wideFrame == "1" then
							WFContainer:SetWidth(350)
						else
							WFContainer:SetWidth(250)
						end
						WFWWPCDB.collapsed = false
					end
				end
			end
		end)
end

WFContainer:RegisterEvent("PLAYER_ENTERING_WORLD")
local f = CreateFrame("Frame")	--need to create a slight delay to get rid of errors and glitches
f:Hide()
f.elapsed = 0
f:SetScript("OnUpdate", function(self, elapsed)
		f.elapsed = f.elapsed + elapsed
		if f.elapsed > .5 then
			SetUpContainer()
			f:Hide()
		end
	end)
WFContainer:SetScript("OnEvent", function()
		Initialize()
		f:Show()
	end)