local OptionsGUI = CreateFrame("FRAME")
local Options = {}
local buttons = {}
local reload = false

---------------------------------
-- UI Element Templates
---------------------------------
-- Global

local function CreatePanel(name, parent)
	local Panel = CreateFrame("FRAME", name.."panel")
	Panel.name = name
	if parent then Panel.parent = parent.name end
	InterfaceOptions_AddCategory(Panel)
	
	return Panel
end

local function CreateText(name, panel, anchor, Xoff, Yoff, text)
	local Text = panel:CreateFontString(name,"ARTWORK","GameFontNormal")
	Text:SetPoint(anchor, Xoff, Yoff)
	Text:SetText(text)	
end

local function CreateTitle(name, panel, anchor, Xoff, Yoff, text)
	local Title = panel:CreateFontString(name,"ARTWORK","GameFontNormalLarge")
	Title:SetPoint(anchor, Xoff, Yoff)
	Title:SetText(text)	
end

---------------------------------
-- Check Button

local function ClickButton(self)
	local isChecked = self:GetChecked()
	isChecked = isChecked or false
	if isChecked and nioConfig[self.categorie][self.key] then return
	elseif not isChecked and not nioConfig[self.categorie][self.key] then return
	else 
		Options[self.categorie] = Options[self.categorie] or {}
		Options[self.categorie][self.key] = isChecked
		if self.needReload then reload = true end
	end
end

local function CreateButton(name, categorie, key, text, panel, anchor, Xoff, Yoff, needReload)
	local Button = CreateFrame("CheckButton", name, panel, "ChatConfigCheckButtonTemplate");
	Button:SetPoint(anchor, Xoff, Yoff)
	Button.categorie = categorie
	Button.key = key
	if needReload then Button.needReload = needReload end
	Button:SetScript("OnClick",ClickButton)
	_G[Button:GetName().."Text"]:SetText(text)		
	table.insert(buttons, name)
end

---------------------------------
-- Edit Box

local function EditBoxEnterPressed(self)
	local EnteredText = self:GetText()
	if EnteredText ~= "" then
		Options[self.categorie] = Options[self.categorie] or {}
		Options[self.categorie][self.key] = EnteredText
		if self.needReload then reload = true end	
	end
end
local function EditBoxOnEditFocusGained(self)
	self:SetText("")	
end
local function EditBoxShow(self)
	self:SetText(nioConfig[self.categorie][self.key])	
end

local function CreateEditBox(name, categorie, key, panel, anchor, Xoff, Yoff, needReload)
	local EditBox = CreateFrame("EditBox", name, panel)
	EditBox:SetPoint(anchor, Xoff, Yoff)
	EditBox.categorie = categorie
	EditBox.key = key
	EditBox.deftext = deftext
	if needReload then EditBox.needReload = needReload end
	EditBox:SetAutoFocus(false)
	EditBox:SetWidth(150)
    EditBox:SetHeight(20)
	EditBox:SetFontObject("GameFontNormal")
	EditBox:SetTextColor(1, 1, 1, 1.0)
	EditBox:SetText("")	
	EditBox:SetScript("OnEnterPressed",EditBoxEnterPressed)	
	EditBox:SetScript("OnShow",EditBoxShow)	
	EditBox:SetScript("OnEditFocusGained",EditBoxOnEditFocusGained)	
end

---------------------------------
-- Slash Button 

local function ClickSlashButton(self)
	local isChecked = self.Checked  or false
	if isChecked then 
		SlashCmdList["NIOUI"](self.paradeactivate)
		_G[self:GetName().."Text"]:SetText(self.textnormal)	
	else 
		SlashCmdList["NIOUI"](self.paraactivate)
		_G[self:GetName().."Text"]:SetText(self.textactivated)	
	end
	if self.needReload then ReloadUI() end
end

local function CreateSlashButton(name, textnormal, textactivated, paraactivate, paradeactivate, panel, anchor, Xoff, Yoff, needReload)
	local Button = CreateFrame("Button", name, panel, "OptionsButtonTemplate");
	Button:SetPoint(anchor, Xoff, Yoff)
	Button.paraactivate = paraactivate
	Button.paradeactivate = paradeactivate
	Button.textnormal = textnormal
	Button.textactivated = textactivated
	if needReload then Button.needReload = needReload end
	Button:SetScript("OnClick",ClickSlashButton)
	_G[Button:GetName().."Text"]:SetText(textnormal)	
end

---------------------------------
-- Save and refresh functions
---------------------------------

local function SaveSettings()
	if _G["CharspecificButton"]:GetChecked() then
		for d,s in pairs(Options) do
			for k,v in pairs(s) do
				nioConfig[d][k] = v
				nioDBC[d] = nioDBC[d] or {}
				nioDBC[d][k] = v
			end				
		end		
	else
		for d,s in pairs(Options) do
			for k,v in pairs(s) do
				nioConfig[d][k] = v
				nioDB[d] = nioDB[d] or {}
				nioDB[d][k] = v
			end				
		end		
	end
end

local function RefreshOptions()
	for i,v in ipairs(buttons) do
		local button = _G[v]
		button:SetChecked(nioConfig[button.categorie][button.key])
	end
end

---------------------------------
-- Make the option menu
---------------------------------
local function MakeOptions()
	
	--Main Panel
	local ConfigurationPanel =  CreatePanel("nioUI")
	CreateTitle("InfoTitle", ConfigurationPanel, "TOPLEFT", 10, -10, "Welcome to nioUI!")	
	CreateText("InfoText", ConfigurationPanel, "TOPLEFT", 10, -50,
		"Here you can set various options for the behaviour of the UI.")	
	local CharButton = CreateFrame("CheckButton", "CharspecificButton", ConfigurationPanel, "ChatConfigCheckButtonTemplate");
	CharButton:SetPoint("TOPRIGHT", -250, -10)
	_G[CharButton:GetName().."Text"]:SetText("Save options for this character only")	
	CreateButton("EnableChatButton", "Chat", "enable", "Enable Chat", ConfigurationPanel, "TOPLEFT", 10, -75, true)
	CreateButton("EnableQuestButton", "Quest", "enable", "Enable Quest Watcher", ConfigurationPanel, "TOPLEFT", 10, -100, true)
	CreateButton("DamageMeterButton", "Misc", "DamageMeter", "Use build in Damage Meter", ConfigurationPanel, "TOPLEFT", 10, -125, true)
	--CreateSlashButton("TipTacAnchorButton", "Tooltip anchor", "Tooltip anchor", "tt", "tt", ConfigurationPanel, "TOPLEFT", 20, -160)
	CreateSlashButton("SkinButton", "Skin addons", "Skin addons", "skin", "skin", ConfigurationPanel, "BOTTOMLEFT", 10, 10, true)
	CreateSlashButton("ReinstallButton", "Reinstall", "Reinstall", "install", "install", ConfigurationPanel, "BOTTOMRIGHT", -10, 10)
	CreateButton("GreyThemeButton", "Theme", "Grey", "Use the grey theme", ConfigurationPanel, "TOPLEFT", 10, -200, true)
	
	--Actionbar Panel
	local ActionbarPanel =  CreatePanel("Actionbar", ConfigurationPanel)
	CreateTitle("ActionbarsTitle", ActionbarPanel, "TOPLEFT", 10, -10, "Actionbars Options")
	CreateButton("EnableActionbarsButton", "Actionbars", "enable", "Enable Actionbars", ActionbarPanel, "TOPLEFT", 10, -35, true)
	CreateSlashButton("HoverBindButton", "Bind Keys", "Bind Keys", "hb", "hb", ActionbarPanel, "TOPLEFT", 10, -65)	
	CreateSlashButton("MoveActionbarsButton", "Unlock Bars", "Lock Bars", "mb", "mb", ActionbarPanel, "TOPLEFT", 200, -65)
	CreateSlashButton("ResetActionbarsButton", "Reset Bars", "Reset Bars", "rb", "rb", ActionbarPanel, "TOPLEFT", 400, -65)	
	CreateButton("ActionbarsAutoEnableButton", "Actionbars", "autoenable", "Enable alle actionbars on load", ActionbarPanel, "TOPLEFT", 10, -100)
	CreateButton("ActionbarsHideHotkeyButton", "Actionbars", "hide_hotkey", "Hide Hotkeys", ActionbarPanel, "TOPLEFT", 10, -125, true)
	CreateButton("ActionbarsHideMacroButton", "Actionbars", "hide_macro_name", "Hide Macro Names", ActionbarPanel, "TOPLEFT", 10, -150, true)
	CreateButton("ShowExtraActionButtonButton", "Actionbars", "show_extraactionbutton", "Show ExtraActionButton", ActionbarPanel, "TOPLEFT", 10, -175, true)
	CreateButton("ActionbarsShowTotembarButton", "Actionbars", "show_totembar", "Show Totembar", ActionbarPanel, "TOPLEFT", 10, -200, true)
	CreateButton("EnableTullaCCButton", "tullaCC", "enable", "Enable Tulla CC", ActionbarPanel, "TOPLEFT", 10, -225)
	CreateButton("MouseoverStance", "Actionbars", "ssmo", "Hide stancebar unless mouseover", ActionbarPanel, "TOPLEFT", 10, -250, true)
	
	--unitframes
	local UnitframesPanel =  CreatePanel("Unitframes", ConfigurationPanel)
	CreateTitle("UnitframesTitle", UnitframesPanel, "TOPLEFT", 10, -10, "Unitframes Options")
	CreateSlashButton("OMFButton", "Move frames", "Move frames", "omf", "omf", UnitframesPanel, "TOPLEFT", 400, -10)
	CreateButton("raidShowSoloButton", "Unitframes", "raidShowSolo", "Show raid frames when solo", UnitframesPanel, "TOPLEFT", 10, -35, true)
	CreateButton("raidShowAllGroupsButton", "Unitframes", "raidShowAllGroups", "Show all raid groups", UnitframesPanel, "TOPLEFT", 10, -60, true)
	CreateButton("classBarButton", "Unitframes", "classBar", "Show classbar", UnitframesPanel, "TOPLEFT", 10, -85, true)
	CreateButton("enableSpecialDebuffButton", "Unitframes", "showSpecialDebuffs", "Show special Debuffs in prominent places", UnitframesPanel, "TOPLEFT", 10, -110, true)
	CreateButton("enableDebuffHighlightButton", "Unitframes", "enableDebuffHighlight", "Highlight dispellable debuffs", UnitframesPanel, "TOPLEFT", 10, -135, true)
	CreateButton("showPlayerClassIndicatorsButton", "Unitframes", "showPlayerClassIndicators", "Show class indicators on raid frames", UnitframesPanel, "TOPLEFT", 10, -160, true)
	CreateButton("showDefCDsIndicatorsButton", "Unitframes", "showDefCDsIndicators", "Show def cd indicators on raid frames", UnitframesPanel, "TOPLEFT", 10, -185, true)
	CreateButton("useRaidDebuffBlacklistButton", "Unitframes", "useRaidDebuffBlacklist", "True: Blacklist, False: Whitelist", UnitframesPanel, "TOPLEFT", 10, -210, true)
	CreateButton("ThreatBarButton", "Unitframes", "ThreatBar", "Show Threatbar", UnitframesPanel, "TOPLEFT", 10, -235, true)
	CreateButton("AltPowerBarButton", "Unitframes", "AltPowerBar", "Show AltPowerbar", UnitframesPanel, "TOPLEFT", 10, -260, true)
	CreateButton("showBossFramesButton", "Unitframes", "showBossFrames", "Show boss frames", UnitframesPanel, "TOPLEFT", 10, -285, true)
	--CreateButton("showArenaFramesButton", "Unitframes", "showArenaFrames", "Show arena frames", UnitframesPanel, "TOPLEFT", 10, -310, true)
	CreateButton("showPortraitsButton", "Unitframes", "showPortraits", "Show portraits on player and target frame", UnitframesPanel, "TOPLEFT", 10, -310, true)

	-- Nameplates
	local NameplatesPanel =  CreatePanel("Nameplates", ConfigurationPanel)
	CreateTitle("NameplatesTitle", NameplatesPanel, "TOPLEFT", 10, -10, "Nameplates Options")
	CreateButton("NameplatesEnableButton", "Nameplates", "enable", "Enable Nameplates", NameplatesPanel, "TOPLEFT", 10, -35, true)
	CreateButton("NPOnlyForTanksButton", "Nameplates", "OnlyForTanks", "Only show threat for tanks", NameplatesPanel, "TOPLEFT", 10, -75, true)
	
	--Map Panel
	local MapPanel =  CreatePanel("Map", ConfigurationPanel)	
	CreateTitle("MinimapTitle", MapPanel, "TOPLEFT", 10, -10, "Minimap Options")	
	CreateButton("EnableMinimapButton", "Map", "enable", "Enable Minimap", MapPanel, "TOPLEFT", 10, -50, true)
	CreateButton("hideInstanceDifficultyButton", "Map", "mmap_hideInstanceDifficulty", "Hide Instance Difficulty", MapPanel, "TOPLEFT", 10, -75, true)
	CreateButton("showCoordinatesButton", "Map", "mmap_showCoordinates", "Show Coordinates", MapPanel, "TOPLEFT", 10, -100, true)
	
	CreateTitle("WorldmapTitle", MapPanel, "TOPLEFT", 10, -125, "Worldmap Options")	
	CreateButton("EnableWorldmapButton", "WorldMap", "enable", "Enable Worldmap", MapPanel, "TOPLEFT", 10, -150, true)

	-- LDB Panel
	local LDBPanel =  CreatePanel("Data Brokers", ConfigurationPanel)
	CreateTitle("LDBTitle", LDBPanel, "TOPLEFT", 10, -10, "Data Brokers Options")
	CreateButton("CargoShipButton", "LDB", "CargoShip", "Enable CargoShip", LDBPanel, "TOPLEFT", 10, -35, true)
	CreateButton("MicroMenuButton", "LDB", "MicroMenu", "Enable MicroMenu Broker", LDBPanel, "TOPLEFT", 10, -75, true)
	CreateButton("ClockButton", "LDB", "Clock", "Enable wClock", LDBPanel, "TOPLEFT", 10, -100, true)
	CreateButton("ClockLocalButton", "LDB", "ClockLocal", "Use local time", LDBPanel, "TOPLEFT", 175, -100)
	CreateButton("DuraButton", "LDB", "Dura", "Enable Durability  Broker", LDBPanel, "TOPLEFT", 10, -125, true)
	CreateButton("MoneyButton", "LDB", "Money", "Enable wMoney", LDBPanel, "TOPLEFT", 10, -150, true)
	CreateButton("FPSButton", "LDB", "FPS", "Enable FPS Broker", LDBPanel, "TOPLEFT", 10, -175, true)
	CreateButton("LatencyButton", "LDB", "Latency", "Enable Latency Broker", LDBPanel, "TOPLEFT", 10, -200, true)
	CreateButton("MemoryButton", "LDB", "Memory", "Show memory usage", LDBPanel, "TOPLEFT", 175, -200, true)
	CreateButton("EquipmentButton", "LDB", "Equipment", "Enable Broker Equipment", LDBPanel, "TOPLEFT", 10, -225, true)
	CreateButton("AddonsButton", "LDB", "Addons", "Enable Addons Broker", LDBPanel, "TOPLEFT", 10, -250, true)
	CreateButton("SpecSwitcherButton", "LDB", "SpecSwitcher", "Enable SpecSwitcher Broker", LDBPanel, "TOPLEFT", 10, -275, true)
	CreateButton("XPButton", "LDB", "XP", "Enable XP Broker", LDBPanel, "TOPLEFT", 10, -300, true)
	CreateButton("GuildButton", "LDB", "Guild", "Enable Guild Broker", LDBPanel, "TOPLEFT", 10, -325, true)
	CreateButton("FriendsButton", "LDB", "Friends", "Enable Friends Broker", LDBPanel, "TOPLEFT", 10, -350, true)
	CreateButton("TradeskillButton", "LDB", "Professions", "Enable Tradeskill Broker", LDBPanel, "TOPLEFT", 10, -375, true)
	CreateButton("RaidButton", "LDB", "Raid", "Enable Raid Broker", LDBPanel, "TOPLEFT", 10, -400, true)
	CreateText("TSText", LDBPanel, "TOPLEFT", 10, -427, "Teamspeak Infos:")	
	CreateEditBox("RaidTSProgButton", "TS", "Program", LDBPanel, "TOPLEFT", 140, -423)
	CreateEditBox("RaidTSIPButton", "TS", "Data", LDBPanel, "TOPLEFT", 300, -423)
	CreateEditBox("RaidTSPWButton", "TS", "PW", LDBPanel, "TOPLEFT", 450, -423)
	
	--Tweaks Panel	
	local TweaksPanel =  CreatePanel("Tweaks", ConfigurationPanel)	
	CreateTitle("TweaksTitle", TweaksPanel, "TOPLEFT", 10, -10, "Tweaks")
	CreateButton("autorepairButton", "Misc", "autorepair", "Autorepair", TweaksPanel, "TOPLEFT", 10, -50, true)
	CreateButton("guildfundsButton", "Misc", "guildfunds", "Use guildfunds", TweaksPanel, "TOPLEFT", 175, -50, true)
	CreateButton("sellgrayButton", "Misc", "sellgray", "Auto sell grays", TweaksPanel, "TOPLEFT", 10, -75, true)
	CreateButton("tekkompareButton", "Misc", "tekkompare", "Enable Tekkompare", TweaksPanel, "TOPLEFT", 10, -100, true)
	CreateButton("CharacterEquipButton", "Misc", "CharacterEquip", "Enable CharacterEquip", TweaksPanel, "TOPLEFT", 10, -125, true)
	CreateButton("SetRoleAutomaticButton", "Misc", "SetRoleAutomatic", "Set Role Automatic", TweaksPanel, "TOPLEFT", 10, -150, true)
	CreateButton("PettbattleHideButton", "Misc", "petbattlehide", "Hide UI-Elements in pet battles", TweaksPanel, "TOPLEFT", 10, -175, true)
	
	RefreshOptions()
			
	ConfigurationPanel.okay = function(self)
		SaveSettings()
		RefreshOptions()
		if reload then
			StaticPopupDialogs["NIOUI_RELOAD"] = {
				text = "Some settings require a reload.\nDo you want to reload now?",
				button1 = OKAY,
				button2 = CANCEL,
				OnAccept = function()
				  ReloadUI()
				end,
				--OnCancel = function() end,
				timeout = 10,
				whileDead = true,
			}	
			StaticPopup_Show ("NIOUI_RELOAD")
		end
	end
	
	ConfigurationPanel.cancel = function(self)
		Options = {}
		RefreshOptions()
	end
	
	ConfigurationPanel.default = function (self)
		nioDB = nil
		nioDBC = nil
		Options = {}
		RefreshOptions()
	end
end

function OptionsGUIOnEvent(self, event)
	if event == "ADDON_LOADED" then
		nioDB = nioDB or {}
		nioDBC = nioDBC or {}
		OptionsGUI:UnregisterEvent("ADDON_LOADED")
	elseif event == "PLAYER_LOGIN" then
		MakeOptions()
		OptionsGUI:UnregisterEvent("PLAYER_LOGIN")
	end
end

OptionsGUI:RegisterEvent("ADDON_LOADED")
OptionsGUI:RegisterEvent("PLAYER_LOGIN")
OptionsGUI:SetScript("OnEvent", OptionsGUIOnEvent) 